@@ -144,7 +144,6 @@ static char *remove_end_comment(char *ptr);
   RETURN
     0  ok
     1  given cinf_file doesn't exist
-    2  out of memory
 
     The global variable 'my_defaults_group_suffix' is updated with value for
     --defaults_group_suffix
@@ -152,7 +151,7 @@ static char *remove_end_comment(char *ptr);
 
 int my_search_option_files(const char *conf_file, int *argc, char ***argv,
                            uint *args_used, Process_option_func func,
-                           void *func_ctx, const char **default_directories)
+                           void *func_ctx)
 {
   const char **dirs, *forced_default_file, *forced_extra_defaults;
   int error= 0;
@@ -183,7 +182,7 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
     /* Handle --defaults-group-suffix= */
     uint i;
     const char **extra_groups;
-    const size_t instance_len= strlen(my_defaults_group_suffix); 
+    const uint instance_len= strlen(my_defaults_group_suffix); 
     struct handle_option_ctx *ctx= (struct handle_option_ctx*) func_ctx;
     char *ptr;
     TYPELIB *group= ctx->group;
@@ -191,16 +190,16 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
     if (!(extra_groups= 
 	  (const char**)alloc_root(ctx->alloc,
                                    (2*group->count+1)*sizeof(char*))))
-      DBUG_RETURN(2);
+      goto err;
     
     for (i= 0; i < group->count; i++)
     {
-      size_t len;
+      uint len;
       extra_groups[i]= group->type_names[i]; /** copy group */
       
       len= strlen(extra_groups[i]);
-      if (!(ptr= alloc_root(ctx->alloc, (uint) (len+instance_len+1))))
-       DBUG_RETURN(2);
+      if (!(ptr= alloc_root(ctx->alloc, len+instance_len+1)))
+	goto err;
       
       extra_groups[i+group->count]= ptr;
       
@@ -255,11 +254,12 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
     }
   }
 
-  DBUG_RETURN(0);
+  DBUG_RETURN(error);
 
 err:
   fprintf(stderr,"Fatal error in defaults handling. Program aborted\n");
-  DBUG_RETURN(1);
+  exit(1);
+  return 0;					/* Keep compiler happy */
 }
 
 
@@ -359,8 +359,9 @@ int get_defaults_options(int argc, char **argv,
   return org_argc - argc;
 }
 
+
 /*
-  Wrapper around my_load_defaults() for interface compatibility.
+  Read options from configurations files
 
   SYNOPSIS
     load_defaults()
@@ -371,35 +372,6 @@ int get_defaults_options(int argc, char **argv,
     argc			Pointer to argc of original program
     argv			Pointer to argv of original program
 
-  NOTES
-
-    This function is NOT thread-safe as it uses a global pointer internally.
-    See also notes for my_load_defaults().
-
-  RETURN
-    0 ok
-    1 The given conf_file didn't exists
-*/
-int load_defaults(const char *conf_file, const char **groups,
-                  int *argc, char ***argv)
-{
-  return my_load_defaults(conf_file, groups, argc, argv, &default_directories);
-}
-
-/*
-  Read options from configurations files
-
-  SYNOPSIS
-    my_load_defaults()
-    conf_file			Basename for configuration file to search for.
-    				If this is a path, then only this file is read.
-    groups			Which [group] entrys to read.
-				Points to an null terminated array of pointers
-    argc			Pointer to argc of original program
-    argv			Pointer to argv of original program
-    default_directories         Pointer to a location where a pointer to the list
-                                of default directories will be stored
-
   IMPLEMENTATION
 
    Read options from configuration files and put them BEFORE the arguments
@@ -414,18 +386,13 @@ int load_defaults(const char *conf_file, const char **groups,
     that was put in *argv
 
    RETURN
-     - If successful, 0 is returned. If 'default_directories' is not NULL,
-     a pointer to the array of default directory paths is stored to a location
-     it points to. That stored value must be passed to my_search_option_files()
-     later.
-     
-     - 1 is returned if the given conf_file didn't exist. In this case, the
-     value pointed to by default_directories is undefined.
+     0	ok
+     1	The given conf_file didn't exists
 */
 
 
-int my_load_defaults(const char *conf_file, const char **groups,
-                  int *argc, char ***argv, const char ***default_directories)
+int load_defaults(const char *conf_file, const char **groups,
+                  int *argc, char ***argv)
 {
   DYNAMIC_ARRAY args;
   TYPELIB group;
@@ -435,11 +402,10 @@ int my_load_defaults(const char *conf_file, const char **groups,
   MEM_ROOT alloc;
   char *ptr,**res;
   struct handle_option_ctx ctx;
-  const char **dirs;
   DBUG_ENTER("load_defaults");
 
   init_alloc_root(&alloc,512,0);
-  if ((dirs= init_default_directories(&alloc)) == NULL)
+  if ((default_directories= init_default_directories(&alloc)) == NULL)
     goto err;
   /*
     Check if the user doesn't want any default option processing
@@ -460,8 +426,6 @@ int my_load_defaults(const char *conf_file, const char **groups,
     (*argc)--;
     *argv=res;
     *(MEM_ROOT*) ptr= alloc;			/* Save alloc root for free */
-    if (default_directories)
-      *default_directories= dirs;
     DBUG_RETURN(0);
   }
 
@@ -480,8 +444,7 @@ int my_load_defaults(const char *conf_file, const char **groups,
   ctx.group= &group;
 
   error= my_search_option_files(conf_file, argc, argv, &args_used,
-                                handle_default_option, (void *) &ctx,
-                                dirs);
+                                handle_default_option, (void *) &ctx);
   /*
     Here error contains <> 0 only if we have a fully specified conf_file
     or a forced default file
@@ -527,10 +490,6 @@ int my_load_defaults(const char *conf_file, const char **groups,
     puts("");
     exit(0);
   }
-
-  if (error == 0 && default_directories)
-    *default_directories= dirs;
-
   DBUG_RETURN(error);
 
  err:
@@ -650,7 +609,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
                                         int recursion_level)
 {
   char name[FN_REFLEN + 10], buff[4096], curr_gr[4096], *ptr, *end, **tmp_ext;
-  char *value, option[4096+2], tmp[FN_REFLEN];
+  char *value, option[4096], tmp[FN_REFLEN];
   static const char includedir_keyword[]= "includedir";
   static const char include_keyword[]= "include";
   const int max_recursion_level= 10;
@@ -936,11 +895,15 @@ void my_print_default_files(const char *conf_file)
     fputs(conf_file,stdout);
   else
   {
-    const char **dirs;
+    /*
+      If default_directories is already initialized, use it.  Otherwise,
+      use a private MEM_ROOT.
+    */
+    const char **dirs = default_directories;
     MEM_ROOT alloc;
     init_alloc_root(&alloc,512,0);
 
-    if ((dirs= init_default_directories(&alloc)) == NULL)
+    if (!dirs && (dirs= init_default_directories(&alloc)) == NULL)
     {
       fputs("Internal error initializing default directories list", stdout);
     }
@@ -995,10 +958,10 @@ void print_defaults(const char *conf_file, const char **groups)
     }
   }
   puts("\nThe following options may be given as the first argument:\n\
---print-defaults        Print the program argument list and exit.\n\
---no-defaults           Don't read default options from any option file.\n\
---defaults-file=#       Only read default options from the given file #.\n\
---defaults-extra-file=# Read this file after the global files are read.");
+--print-defaults	Print the program argument list and exit\n\
+--no-defaults		Don't read default options from any options file\n\
+--defaults-file=#	Only read default options from the given file #\n\
+--defaults-extra-file=# Read this file after the global files are read");
 }
 
 #include <help_end.h>
@@ -1007,7 +970,7 @@ void print_defaults(const char *conf_file, const char **groups)
 static int add_directory(MEM_ROOT *alloc, const char *dir, const char **dirs)
 {
   char buf[FN_REFLEN];
-  size_t len;
+  uint len;
   char *p;
   my_bool err __attribute__((unused));
 
@@ -1041,14 +1004,14 @@ static size_t my_get_system_windows_directory(char *buffer, size_t size)
                                              "GetSystemWindowsDirectoryA");
 
   if (func_ptr)
-    return func_ptr(buffer, (uint) size);
+    return func_ptr(buffer, size);
 
   /*
     Windows NT 4.0 Terminal Server Edition:  
     To retrieve the shared Windows directory, call GetSystemDirectory and
     trim the "System32" element from the end of the returned path.
   */
-  count= GetSystemDirectory(buffer, (uint) size);
+  count= GetSystemDirectory(buffer, size);
   if (count > 8 && stricmp(buffer+(count-8), "\\System32") == 0)
   {
     count-= 8;
@@ -1127,7 +1090,7 @@ static const char **init_default_directories(MEM_ROOT *alloc)
   errors += add_directory(alloc, "/etc/mysql/", dirs);
 
 #if defined(DEFAULT_SYSCONFDIR)
-  if (DEFAULT_SYSCONFDIR[0])
+  if (DEFAULT_SYSCONFDIR != "")
     errors += add_directory(alloc, DEFAULT_SYSCONFDIR, dirs);
 #endif /* DEFAULT_SYSCONFDIR */
 
