@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +13,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-#define USES_TYPES				/* sys/types is included */
 #include "mysys_priv.h"
 #include <sys/stat.h>
 #include <m_string.h>
@@ -29,21 +27,22 @@ struct utimbuf {
 };
 #endif
 
-	/* Append a file to another */
+/*
+  Append a file to another
+
+  NOTES
+    Don't set MY_FNABP or MY_NABP bits on when calling this function
+*/
 
 int my_append(const char *from, const char *to, myf MyFlags)
-
-
-				/* Dont set MY_FNABP or MY_NABP bits on
-					   when calling this funktion */
 {
   uint Count;
   File from_file,to_file;
-  char buff[IO_SIZE];
+  uchar buff[IO_SIZE];
   DBUG_ENTER("my_append");
   DBUG_PRINT("my",("from %s to %s MyFlags %d", from, to, MyFlags));
 
-  from_file=to_file= -1;
+  from_file= to_file= -1;
 
   if ((from_file=my_open(from,O_RDONLY,MyFlags)) >= 0)
   {
