@@ -20,9 +20,8 @@
 #include <mysys_err.h>
 #include <my_getopt.h>
 #include <errno.h>
-#include <m_string.h>
 
-typedef void (*init_func_p)(const struct my_option *option, void *variable,
+typedef void (*init_func_p)(const struct my_option *option, uchar* *variable,
                             longlong value);
 
 static void default_reporter(enum loglevel level, const char *format, ...);
@@ -40,11 +39,11 @@ static ulonglong getopt_ull(char *arg, const struct my_option *optp,
 static double getopt_double(char *arg, const struct my_option *optp, int *err);
 static void init_variables(const struct my_option *options,
                            init_func_p init_one_value);
-static void init_one_value(const struct my_option *option, void *variable,
+static void init_one_value(const struct my_option *option, uchar* *variable,
 			   longlong value);
-static void fini_one_value(const struct my_option *option, void *variable,
+static void fini_one_value(const struct my_option *option, uchar* *variable,
 			   longlong value);
-static int setval(const struct my_option *opts, void *value, char *argument,
+static int setval(const struct my_option *opts, uchar* *value, char *argument,
 		  my_bool set_maximum_value);
 static char *check_struct_option(char *cur_arg, char *key_name);
 
@@ -101,9 +100,10 @@ static void default_reporter(enum loglevel level,
   one. Call function 'get_one_option()' once for each option.
 */
 
-static my_getopt_value getopt_get_addr;
+static uchar** (*getopt_get_addr)(const char *, uint, const struct my_option *);
 
-void my_getopt_register_get_addr(my_getopt_value func_addr)
+void my_getopt_register_get_addr(uchar** (*func_addr)(const char *, uint,
+						    const struct my_option *))
 {
   getopt_get_addr= func_addr;
 }
@@ -115,10 +115,10 @@ int handle_options(int *argc, char ***argv,
   uint opt_found, argvpos= 0, length;
   my_bool end_of_options= 0, must_be_var, set_maximum_value,
           option_is_loose;
-  char **pos, **pos_end, *optend, *UNINIT_VAR(prev_found),
+  char **pos, **pos_end, *optend, *prev_found,
        *opt_str, key_name[FN_REFLEN];
   const struct my_option *optp;
-  void *value;
+  uchar* *value;
   int error, i;
 
   LINT_INIT(opt_found);
@@ -145,10 +145,6 @@ int handle_options(int *argc, char ***argv,
       {                                       /* --set-variable, or -O  */
 	if (*cur_arg == 'O')
 	{
-          my_getopt_error_reporter(WARNING_LEVEL, 
-                                   "%s: Option '-O' is deprecated. "
-                                   "Use --variable-name=value instead.",
-                                   my_progname);
 	  must_be_var= 1;
 
 	  if (!(*++cur_arg))	/* If not -Ovar=# */
@@ -168,11 +164,6 @@ int handle_options(int *argc, char ***argv,
 	}
 	else if (!getopt_compare_strings(cur_arg, "-set-variable", 13))
 	{
-          my_getopt_error_reporter(WARNING_LEVEL, 
-                                   "%s: Option '--set-variable' is deprecated. "
-                                   "Use --variable-name=value instead.",
-                                   my_progname);
-
 	  must_be_var= 1;
 	  if (cur_arg[13] == '=')
 	  {
@@ -371,13 +362,9 @@ int handle_options(int *argc, char ***argv,
                                      my_progname, optp->name);
 	  return EXIT_NO_ARGUMENT_ALLOWED;
 	}
-        error= 0;
 	value= optp->var_type & GET_ASK_ADDR ?
-	  (*getopt_get_addr)(key_name, (uint) strlen(key_name), optp, &error) :
-          optp->value;
-        if (error)
-          return error;
-
+	  (*getopt_get_addr)(key_name, (uint) strlen(key_name), optp) : optp->value;
+  
 	if (optp->arg_type == NO_ARG)
 	{
 	  if (optend && (optp->var_type & GET_TYPE_MASK) != GET_BOOL)
@@ -405,33 +392,31 @@ int handle_options(int *argc, char ***argv,
 	    else
 	    {
 	      my_getopt_error_reporter(WARNING_LEVEL,
-				       "%s: ignoring option '--%s' due to "
-                                       "invalid value '%s'",
+				       "%s: ignoring option '--%s' due to \
+invalid value '%s'",
 				       my_progname, optp->name, optend);
 	      continue;
 	    }
-	    if (get_one_option(optp->id, optp,
-                               *((my_bool*) value) ?
-                               (char*) "1" : disabled_my_option))
-              return EXIT_ARGUMENT_INVALID;
+	    get_one_option(optp->id, optp,
+			   *((my_bool*) value) ?
+			   (char*) "1" : disabled_my_option);
 	    continue;
 	  }
 	  argument= optend;
 	}
 	else if (optp->arg_type == OPT_ARG &&
-		 (((optp->var_type & GET_TYPE_MASK) == GET_BOOL) ||
-                   (optp->var_type & GET_TYPE_MASK) == GET_ENUM))
-  {
-    if (optend == disabled_my_option)
-      init_one_value(optp, value, 0);
-    else
-    {
-      if (!optend) /* No argument -> enable option */
-        init_one_value(optp, value, 1);
-      else
-        argument= optend;
-    }
-  }
+		 (optp->var_type & GET_TYPE_MASK) == GET_BOOL)
+	{
+	  if (optend == disabled_my_option)
+	    *((my_bool*) value)= (my_bool) 0;
+	  else
+	  {
+	    if (!optend) /* No argument -> enable option */
+	      *((my_bool*) value)= (my_bool) 1;
+            else
+              argument= optend;
+	  }
+	}
 	else if (optp->arg_type == REQUIRED_ARG && !optend)
 	{
 	  /* Check if there are more arguments after this one */
@@ -472,8 +457,7 @@ int handle_options(int *argc, char ***argv,
 		  optp->arg_type == NO_ARG)
 	      {
 		*((my_bool*) optp->value)= (my_bool) 1;
-		if (get_one_option(optp->id, optp, argument))
-                  return EXIT_UNSPECIFIED_ERROR;
+		get_one_option(optp->id, optp, argument);
 		continue;
 	      }
 	      else if (optp->arg_type == REQUIRED_ARG ||
@@ -492,8 +476,7 @@ int handle_options(int *argc, char ***argv,
                   {
                     if (optp->var_type == GET_BOOL)
                       *((my_bool*) optp->value)= (my_bool) 1;
-                    if (get_one_option(optp->id, optp, argument))
-                      return EXIT_UNSPECIFIED_ERROR;
+                    get_one_option(optp->id, optp, argument);
                     continue;
                   }
 		  /* Check if there are more arguments after this one */
@@ -518,8 +501,7 @@ int handle_options(int *argc, char ***argv,
                                          my_progname, argument, optp->name);
 		return error;
 	      }
-	      if (get_one_option(optp->id, optp, argument))
-                return EXIT_UNSPECIFIED_ERROR;
+	      get_one_option(optp->id, optp, argument);
 	      break;
 	    }
 	  }
@@ -542,8 +524,7 @@ int handle_options(int *argc, char ***argv,
                                  my_progname, argument, optp->name);
 	return error;
       }
-      if (get_one_option(optp->id, optp, argument))
-        return EXIT_UNSPECIFIED_ERROR;
+      get_one_option(optp->id, optp, argument);
 
       (*argc)--; /* option handled (short or long), decrease argument count */
     }
@@ -610,14 +591,15 @@ static char *check_struct_option(char *cur_arg, char *key_name)
   Will set the option value to given value
 */
 
-static int setval(const struct my_option *opts, void *value, char *argument,
+static int setval(const struct my_option *opts, uchar* *value, char *argument,
 		  my_bool set_maximum_value)
 {
   int err= 0;
 
   if (value && argument)
   {
-    void *result_pos= ((set_maximum_value) ? opts->u_max_value : value);
+    uchar* *result_pos= ((set_maximum_value) ?
+		       opts->u_max_value : value);
 
     if (!result_pos)
       return EXIT_NO_PTR_TO_VARIABLE;
@@ -657,18 +639,8 @@ static int setval(const struct my_option *opts, void *value, char *argument,
 	return EXIT_OUT_OF_MEMORY;
       break;
     case GET_ENUM:
-      if (((*(int*)result_pos)=
-             find_type(argument, opts->typelib, 2) - 1) < 0)
-      {
-        /*
-          Accept an integer representation of the enumerated item.
-        */
-        char *endptr;
-        unsigned int arg= (unsigned int) strtol(argument, &endptr, 10);
-        if (*endptr || arg >= opts->typelib->count)
-          return EXIT_ARGUMENT_INVALID;
-        *(int*)result_pos= arg;
-      }
+      if (((*(int*)result_pos)= find_type(argument, opts->typelib, 2) - 1) < 0)
+        return EXIT_ARGUMENT_INVALID;
       break;
     case GET_SET:
       *((ulonglong*)result_pos)= find_typeset(argument, opts->typelib, &err);
@@ -865,8 +837,7 @@ longlong getopt_ll_limit_value(longlong num, const struct my_option *optp,
   if (num < optp->min_value)
   {
     num= optp->min_value;
-    if (old < optp->min_value)
-      adjusted= TRUE;
+    adjusted= TRUE;
   }
 
   if (fix)
@@ -937,8 +908,7 @@ ulonglong getopt_ull_limit_value(ulonglong num, const struct my_option *optp,
   if (num < (ulonglong) optp->min_value)
   {
     num= (ulonglong) optp->min_value;
-    if (old < (ulonglong) optp->min_value)
-      adjusted= TRUE;
+    adjusted= TRUE;
   }
 
   if (fix)
@@ -992,7 +962,7 @@ static double getopt_double(char *arg, const struct my_option *optp, int *err)
     value		Pointer to variable
 */
 
-static void init_one_value(const struct my_option *option, void *variable,
+static void init_one_value(const struct my_option *option, uchar* *variable,
 			   longlong value)
 {
   DBUG_ENTER("init_one_value");
@@ -1001,28 +971,24 @@ static void init_one_value(const struct my_option *option, void *variable,
     *((my_bool*) variable)= (my_bool) value;
     break;
   case GET_INT:
-    *((int*) variable)= (int) getopt_ll_limit_value((int) value, option, NULL);
+    *((int*) variable)= (int) value;
     break;
+  case GET_UINT:
   case GET_ENUM:
     *((uint*) variable)= (uint) value;
     break;
-  case GET_UINT:
-    *((uint*) variable)= (uint) getopt_ull_limit_value((uint) value, option, NULL);
-    break;
   case GET_LONG:
-    *((long*) variable)= (long) getopt_ll_limit_value((long) value, option, NULL);
+    *((long*) variable)= (long) value;
     break;
   case GET_ULONG:
-    *((ulong*) variable)= (ulong) getopt_ull_limit_value((ulong) value, option, NULL);
+    *((ulong*) variable)= (ulong) value;
     break;
   case GET_LL:
-    *((longlong*) variable)= (longlong) getopt_ll_limit_value((longlong) value, option, NULL);
+    *((longlong*) variable)= (longlong) value;
     break;
   case GET_ULL:
-    *((ulonglong*) variable)= (ulonglong) getopt_ull_limit_value((ulonglong) value, option, NULL);
-    break;
   case GET_SET:
-    *((ulonglong*) variable)= (ulonglong) value;
+    *((ulonglong*) variable)=  (ulonglong) value;
     break;
   case GET_DOUBLE:
     *((double*) variable)=  (double) value;
@@ -1066,7 +1032,7 @@ static void init_one_value(const struct my_option *option, void *variable,
     value		Pointer to variable
 */
 
-static void fini_one_value(const struct my_option *option, void *variable,
+static void fini_one_value(const struct my_option *option, uchar* *variable,
 			   longlong value __attribute__ ((unused)))
 {
   DBUG_ENTER("fini_one_value");
@@ -1107,7 +1073,7 @@ static void init_variables(const struct my_option *options,
   DBUG_ENTER("init_variables");
   for (; options->name; options++)
   {
-    void *variable;
+    uchar* *variable;
     DBUG_PRINT("options", ("name: '%s'", options->name));
     /*
       We must set u_max_value first as for some variables
@@ -1119,7 +1085,7 @@ static void init_variables(const struct my_option *options,
     if (options->value)
       init_one_value(options, options->value, options->def_value);
     if (options->var_type & GET_ASK_ADDR &&
-	(variable= (*getopt_get_addr)("", 0, options, 0)))
+	(variable= (*getopt_get_addr)("", 0, options)))
       init_one_value(options, variable, options->def_value);
   }
   DBUG_VOID_RETURN;
@@ -1222,8 +1188,8 @@ void my_print_variables(const struct my_option *options)
   printf("--------------------------------- -----------------------------\n");
   for (optp= options; optp->id; optp++)
   {
-    void *value= (optp->var_type & GET_ASK_ADDR ?
-		  (*getopt_get_addr)("", 0, optp, 0) : optp->value);
+    uchar* *value= (optp->var_type & GET_ASK_ADDR ?
+		  (*getopt_get_addr)("", 0, optp) : optp->value);
     if (value)
     {
       printf("%s ", optp->name);
