@@ -1,10 +1,9 @@
 #!/bin/sh
-# Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+# Copyright (C) 2000-2006 MySQL AB
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# the Free Software Foundation; version 2 of the License.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -88,14 +87,21 @@ pkglibdir='@pkglibdir@'
 pkglibdir_rel=`echo $pkglibdir | sed -e "s;^$basedir/;;"`
 fix_path pkglibdir $pkglibdir_rel lib/mysql lib
 
+plugindir='@pkgplugindir@'
+
 pkgincludedir='@pkgincludedir@'
 fix_path pkgincludedir include/mysql include
 
 version='@VERSION@'
 socket='@MYSQL_UNIX_ADDR@'
-port='@MYSQL_TCP_PORT@'
 ldflags='@LDFLAGS@'
 
+if [ @MYSQL_TCP_PORT_DEFAULT@ -eq 0 ]; then
+  port=0
+else
+  port=@MYSQL_TCP_PORT@
+fi
+
 # Create options 
 # We intentionally add a space to the beginning and end of lib strings, simplifies replace later
 libs=" $ldflags -L$pkglibdir -lmysqlclient @ZLIB_DEPS@ @NON_THREADED_LIBS@"
@@ -103,16 +109,29 @@ libs="$libs @openssl_libs@ @STATIC_NSS_FLAGS@ "
 libs_r=" $ldflags -L$pkglibdir -lmysqlclient_r @ZLIB_DEPS@ @LIBS@ @openssl_libs@ "
 embedded_libs=" $ldflags -L$pkglibdir -lmysqld @ZLIB_DEPS@ @LIBS@ @WRAPLIBS@ @innodb_system_libs@ @openssl_libs@ "
 
+if [ -r "$pkglibdir/libmygcc.a" ]; then
+  # When linking against the static library with a different version of GCC
+  # from what was used to compile the library, some symbols may not be defined
+  # automatically.  We package the libmygcc.a from the build host, to provide
+  # definitions for those.  Bugs 4921, 19561, 19817, 21158, etc.
+  libs="$libs -lmygcc "
+  libs_r="$libs_r -lmygcc "
+  embedded_libs="$embedded_libs -lmygcc "
+fi
+
 cflags="-I$pkgincludedir @CFLAGS@ " #note: end space!
 include="-I$pkgincludedir"
 
 # Remove some options that a client doesn't have to care about
 # FIXME until we have a --cxxflags, we need to remove -Xa
 #       and -xstrconst to make --cflags usable for Sun Forte C++
+# FIXME until we have a --cxxflags, we need to remove -AC99
+#       to make --cflags usable for HP C++ (aCC)
 for remove in DDBUG_OFF DSAFEMALLOC USAFEMALLOC DSAFE_MUTEX \
               DPEDANTIC_SAFEMALLOC DUNIV_MUST_NOT_INLINE DFORCE_INIT_OF_VARS \
               DEXTRA_DEBUG DHAVE_purify O 'O[0-9]' 'xO[0-9]' 'W[-A-Za-z]*' \
-              Xa xstrconst "xc99=none" \
+              'mtune=[-A-Za-z0-9]*' 'mcpu=[-A-Za-z0-9]*' 'march=[-A-Za-z0-9]*' \
+              Xa xstrconst "xc99=none" AC99 \
               unroll2 ip mp restrict
 do
   # The first option we might strip will always have a space before it because
@@ -122,7 +141,7 @@ done
 cflags=`echo "$cflags"|sed -e 's/ *\$//'` 
 
 # Same for --libs(_r)
-for remove in lmtmalloc static-libcxa i-static
+for remove in lmtmalloc static-libcxa i-static static-intel
 do
   # We know the strings starts with a space
   libs=`echo "$libs"|sed -e "s/ -$remove  */ /g"` 
@@ -143,6 +162,7 @@ Options:
         --include        [$include]
         --libs           [$libs]
         --libs_r         [$libs_r]
+        --plugindir      [$plugindir]
         --socket         [$socket]
         --port           [$port]
         --version        [$version]
@@ -159,6 +179,7 @@ while test $# -gt 0; do
         --include) echo "$include" ;;
         --libs)    echo "$libs" ;;
         --libs_r)  echo "$libs_r" ;;
+        --plugindir) echo "$plugindir" ;;
         --socket)  echo "$socket" ;;
         --port)    echo "$port" ;;
         --version) echo "$version" ;;
