@@ -1,6 +1,5 @@
 #!/bin/sh
-# Copyright (c) 2000-2008 MySQL AB, 2009, 2010 Sun Microsystems, Inc.
-# Use is subject to license terms.
+# Copyright (C) 2002-2003 MySQL AB
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +21,6 @@
 basedir=""
 builddir=""
 ldata="@localstatedir@"
-langdir=""
 srcdir=""
 
 args=""
@@ -46,15 +44,10 @@ Usage: $0 [OPTIONS]
   --cross-bootstrap    For internal use.  Used when building the MySQL system
                        tables on a different host than the target.
   --datadir=path       The path to the MySQL data directory.
-  --defaults-extra-file=name
-                       Read this file after the global files are read.
-  --defaults-file=name Only read default options from the given file name.
   --force              Causes mysql_install_db to run even if DNS does not
                        work.  In that case, grant table entries that normally
                        use hostnames will use IP addresses.
-  --help               Display this help and exit.                     
   --ldata=path         The path to the MySQL data directory. Same as --datadir.
-  --no-defaults        Don't read default options from any option file.
   --rpm                For internal use.  This option is used by RPM files
                        during the MySQL installation process.
   --skip-name-resolve  Use IP addresses rather than hostnames when creating
@@ -113,7 +106,7 @@ parse_arguments()
         # Note that the user will be passed to mysqld so that it runs
         # as 'user' (crucial e.g. if log-bin=/some_other_path/
         # where a chown of datadir won't help)
-        user=`parse_arg "$arg"` ;;
+	 user=`parse_arg "$arg"` ;;
       --skip-name-resolve) ip_only=1 ;;
       --verbose) verbose=1 ;; # Obsolete
       --rpm) in_rpm=1 ;;
@@ -178,20 +171,7 @@ find_in_basedir()
 cannot_find_file()
 {
   echo
-  echo "FATAL ERROR: Could not find $1"
-
-  shift
-  if test $# -ne 0
-  then
-    echo
-    echo "The following directories were searched:"
-    echo
-    for dir in "$@"
-    do
-      echo "    $dir"
-    done
-  fi
-
+  echo "FATAL ERROR: Could not find $*"
   echo
   echo "If you compiled from source, you need to run 'make install' to"
   echo "copy the software into the correct location ready for operation."
@@ -230,11 +210,6 @@ then
 elif test -n "$basedir"
 then
   print_defaults=`find_in_basedir my_print_defaults bin extra`
-  if test -z "$print_defaults"
-  then
-    cannot_find_file my_print_defaults $basedir/bin $basedir/extra
-    exit 1
-  fi
 else
   print_defaults="@bindir@/my_print_defaults"
 fi
@@ -257,7 +232,7 @@ then
   bindir="$basedir/client"
   extra_bindir="$basedir/extra"
   mysqld="$basedir/sql/mysqld"
-  langdir="$srcdir/sql/share/english"
+  mysqld_opt="--language=$srcdir/sql/share/english"
   pkgdatadir="$srcdir/scripts"
   scriptdir="$srcdir/scripts"
 elif test -n "$basedir"
@@ -265,23 +240,7 @@ then
   bindir="$basedir/bin"
   extra_bindir="$bindir"
   mysqld=`find_in_basedir mysqld libexec sbin bin`
-  if test -z "$mysqld"
-  then
-    cannot_find_file mysqld $basedir/libexec $basedir/sbin $basedir/bin
-    exit 1
-  fi
-  langdir=`find_in_basedir --dir errmsg.sys share/english share/mysql/english`
-  if test -z "$langdir"
-  then
-    cannot_find_file errmsg.sys $basedir/share/english $basedir/share/mysql/english
-    exit 1
-  fi
   pkgdatadir=`find_in_basedir --dir fill_help_tables.sql share share/mysql`
-  if test -z "$pkgdatadir"
-  then
-    cannot_find_file fill_help_tables.sql $basedir/share $basedir/share/mysql
-    exit 1
-  fi
   scriptdir="$basedir/scripts"
 else
   basedir="@prefix@"
@@ -312,16 +271,6 @@ then
   exit 1
 fi
 
-if test -n "$langdir"
-then
-  if test ! -f "$langdir/errmsg.sys"
-  then
-    cannot_find_file "$langdir/errmsg.sys"
-    exit 1
-  fi
-  mysqld_opt="--language=$langdir"
-fi
-
 # Try to determine the hostname
 hostname=`@HOSTNAME@`
 
@@ -416,7 +365,8 @@ else
   echo "Try 'mysqld --help' if you have problems with paths.  Using --log"
   echo "gives you a log in $ldata that may be helpful."
   echo
-  echo "Please consult the MySQL manual section"
+  echo "The latest information about MySQL is available on the web at"
+  echo "http://www.mysql.com/.  Please consult the MySQL manual section"
   echo "'Problems running mysql_install_db', and the manual section that"
   echo "describes problems on your OS.  Another information source are the"
   echo "MySQL email archives available at http://lists.mysql.com/."
@@ -475,6 +425,9 @@ then
   echo
   echo "Please report any problems with the $scriptdir/mysqlbug script!"
   echo
+  echo "The latest information about MySQL is available at http://www.mysql.com/"
+  echo "Support MySQL by buying support/licenses from http://shop.mysql.com/"
+  echo
 fi
 
 exit 0
