@@ -1,5 +1,4 @@
-/*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (C) 2000-2003 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
   This file is included by both libmysql.c (the MySQL client C API)
@@ -122,7 +120,6 @@ const char 	*def_shared_memory_base_name= default_shared_memory_base_name;
 
 static void mysql_close_free_options(MYSQL *mysql);
 static void mysql_close_free(MYSQL *mysql);
-static void mysql_prune_stmt_list(MYSQL *mysql);
 
 #if !(defined(__WIN__) || defined(__NETWARE__))
 static int wait_for_data(my_socket fd, uint timeout);
@@ -391,7 +388,7 @@ HANDLE create_named_pipe(MYSQL *mysql, uint connect_timeout, char **arg_host,
 			    0,
 			    NULL,
 			    OPEN_EXISTING,
-			    FILE_FLAG_OVERLAPPED,
+			    0,
 			    NULL )) != INVALID_HANDLE_VALUE)
       break;
     if (GetLastError() != ERROR_PIPE_BUSY)
@@ -479,18 +476,6 @@ HANDLE create_shared_memory(MYSQL *mysql,NET *net, uint connect_timeout)
   DWORD error_code = 0;
   DWORD event_access_rights= SYNCHRONIZE | EVENT_MODIFY_STATE;
   char *shared_memory_base_name = mysql->options.shared_memory_base_name;
-  static const char *name_prefixes[] = {"","Global\\"};
-  const char *prefix;
-  int i;
-
-  /*
-    If this is NULL, somebody freed the MYSQL* options.  mysql_close()
-    is a good candidate.  We don't just silently (re)set it to
-    def_shared_memory_base_name as that would create really confusing/buggy
-    behavior if the user passed in a different name on the command-line or
-    in a my.cnf.
-  */
-  DBUG_ASSERT(shared_memory_base_name != NULL);
 
   /*
      get enough space base-name + '_' + longest suffix we might ever send
@@ -505,18 +490,9 @@ HANDLE create_shared_memory(MYSQL *mysql,NET *net, uint connect_timeout)
     shared_memory_base_name is unique value for each server
     unique_part is uniquel value for each object (events and file-mapping)
   */
-  for (i = 0; i< array_elements(name_prefixes); i++)
-  {
-    prefix= name_prefixes[i];
-    suffix_pos = strxmov(tmp, prefix , shared_memory_base_name, "_", NullS);
-    strmov(suffix_pos, "CONNECT_REQUEST");
-    event_connect_request= OpenEvent(event_access_rights, FALSE, tmp);
-    if (event_connect_request)
-    {
-      break;
-    }
-  }
-  if (!event_connect_request)
+  suffix_pos = strxmov(tmp, "Global\\", shared_memory_base_name, "_", NullS);
+  strmov(suffix_pos, "CONNECT_REQUEST");
+  if (!(event_connect_request= OpenEvent(event_access_rights, FALSE, tmp)))
   {
     error_allow = CR_SHARED_MEMORY_CONNECT_REQUEST_ERROR;
     goto err;
@@ -568,7 +544,7 @@ HANDLE create_shared_memory(MYSQL *mysql,NET *net, uint connect_timeout)
     unique_part is uniquel value for each object (events and file-mapping)
     number_of_connection is number of connection between server and client
   */
-  suffix_pos = strxmov(tmp, prefix , shared_memory_base_name, "_", connect_number_char,
+  suffix_pos = strxmov(tmp, "Global\\", shared_memory_base_name, "_", connect_number_char,
 		       "_", NullS);
   strmov(suffix_pos, "DATA");
   if ((handle_file_map = OpenFileMapping(FILE_MAP_WRITE,FALSE,tmp)) == NULL)
@@ -625,7 +601,7 @@ HANDLE create_shared_memory(MYSQL *mysql,NET *net, uint connect_timeout)
 err2:
   if (error_allow == 0)
   {
-    net->vio= vio_new_win32shared_memory(handle_file_map,handle_map,
+    net->vio= vio_new_win32shared_memory(net,handle_file_map,handle_map,
                                          event_server_wrote,
                                          event_server_read,event_client_wrote,
                                          event_client_read,event_conn_closed);
@@ -944,14 +920,10 @@ void end_server(MYSQL *mysql)
   {
     init_sigpipe_variables
     DBUG_PRINT("info",("Net: %s", vio_description(mysql->net.vio)));
-#ifdef MYSQL_SERVER
-    slave_io_thread_detach_vio();
-#endif
     set_sigpipe(mysql);
     vio_delete(mysql->net.vio);
     reset_sigpipe(mysql);
     mysql->net.vio= 0;          /* Marker */
-    mysql_prune_stmt_list(mysql);
   }
   net_end(&mysql->net);
   free_old_query(mysql);
@@ -1049,7 +1021,7 @@ void mysql_read_default_options(struct st_mysql_options *options,
   argc=1; argv=argv_buff; argv_buff[0]= (char*) "client";
   groups[0]= (char*) "client"; groups[1]= (char*) group; groups[2]=0;
 
-  my_load_defaults(filename, groups, &argc, &argv, NULL);
+  load_defaults(filename, groups, &argc, &argv);
   if (argc != 1)				/* If some default option */
   {
     char **option=argv;
@@ -1860,7 +1832,7 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
 		       uint port, const char *unix_socket,ulong client_flag)
 {
   char		buff[NAME_LEN+USERNAME_LENGTH+100];
-  char		*end,*host_info= NULL;
+  char		*end,*host_info;
   my_socket	sock;
   in_addr_t	ip_addr;
   struct	sockaddr_in sock_addr;
@@ -1878,6 +1850,7 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
 #endif
   init_sigpipe_variables
   DBUG_ENTER("mysql_real_connect");
+  LINT_INIT(host_info);
 
   DBUG_PRINT("enter",("host: %s  db: %s  user: %s",
 		      host ? host : "(Null)",
@@ -2030,7 +2003,7 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
     }
     else
     {
-      net->vio= vio_new_win32pipe(hPipe);
+      net->vio=vio_new_win32pipe(hPipe);
       my_snprintf(host_info=buff, sizeof(buff)-1,
                   ER(CR_NAMEDPIPE_CONNECTION), unix_socket);
     }
@@ -2040,7 +2013,6 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
       (!mysql->options.protocol ||
        mysql->options.protocol == MYSQL_PROTOCOL_TCP))
   {
-    int status= -1;
     unix_socket=0;				/* This is not used */
     if (!port)
       port=mysql_port;
@@ -2066,7 +2038,6 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
     net->vio= vio_new(sock, VIO_TYPE_TCPIP, VIO_BUFFERED_READ);
     bzero((char*) &sock_addr,sizeof(sock_addr));
     sock_addr.sin_family = AF_INET;
-    sock_addr.sin_port = (ushort) htons((ushort) port);
 
     /*
       The server name may be a host name or IP address
@@ -2075,46 +2046,28 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
     if ((int) (ip_addr = inet_addr(host)) != (int) INADDR_NONE)
     {
       memcpy_fixed(&sock_addr.sin_addr,&ip_addr,sizeof(ip_addr));
-      status= my_connect(sock, (struct sockaddr *) &sock_addr,
-                         sizeof(sock_addr), mysql->options.connect_timeout);
     }
     else
     {
-      int i, tmp_errno;
+      int tmp_errno;
       struct hostent tmp_hostent,*hp;
       char buff2[GETHOSTBYNAME_BUFF_SIZE];
       hp = my_gethostbyname_r(host,&tmp_hostent,buff2,sizeof(buff2),
 			      &tmp_errno);
-
-      /*
-        Don't attempt to connect to non IPv4 addresses as the client could
-        end up sending information to a unknown server. For example, a IPv6
-        address might be returned from gethostbyname depending on options
-        set via the RES_OPTIONS environment variable.
-      */
-      if (!hp || (hp->h_addrtype != AF_INET))
+      if (!hp)
       {
 	my_gethostbyname_r_free();
         set_mysql_extended_error(mysql, CR_UNKNOWN_HOST, unknown_sqlstate,
                                  ER(CR_UNKNOWN_HOST), host, tmp_errno);
 	goto error;
       }
-
-      for (i= 0; status && hp->h_addr_list[i]; i++)
-      {
-        IF_DBUG(char ipaddr[18];)
-        memcpy(&sock_addr.sin_addr, hp->h_addr_list[i],
-               min(sizeof(sock_addr.sin_addr), (size_t) hp->h_length));
-        DBUG_PRINT("info",("Trying %s...",
-                          (my_inet_ntoa(sock_addr.sin_addr, ipaddr), ipaddr)));
-        status= my_connect(sock, (struct sockaddr *) &sock_addr,
-                           sizeof(sock_addr), mysql->options.connect_timeout);
-      }
-
+      memcpy(&sock_addr.sin_addr, hp->h_addr,
+             min(sizeof(sock_addr.sin_addr), (size_t) hp->h_length));
       my_gethostbyname_r_free();
     }
-
-    if (status)
+    sock_addr.sin_port = (ushort) htons((ushort) port);
+    if (my_connect(sock,(struct sockaddr *) &sock_addr, sizeof(sock_addr),
+		   mysql->options.connect_timeout))
     {
       DBUG_PRINT("error",("Got error %d on connect to '%s'",socket_errno,
 			  host));
@@ -2457,11 +2410,6 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
     goto error;
   }
 
-  /*
-     Using init_commands is not supported when connecting from within the
-     server.
-  */
-#ifndef MYSQL_SERVER
   if (mysql->options.init_commands)
   {
     DYNAMIC_ARRAY *init_commands= mysql->options.init_commands;
@@ -2473,26 +2421,18 @@ CLI_MYSQL_REAL_CONNECT(MYSQL *mysql,const char *host, const char *user,
 
     for (; ptr < end_command; ptr++)
     {
-      int status;
-
+      MYSQL_RES *res;
       if (mysql_real_query(mysql,*ptr, (ulong) strlen(*ptr)))
 	goto error;
-
-      do {
-        if (mysql->fields)
-        {
-          MYSQL_RES *res;
-          if (!(res= cli_use_result(mysql)))
-            goto error;
-          mysql_free_result(res);
-        }
-        if ((status= mysql_next_result(mysql)) > 0)
-          goto error;
-      } while (status == 0);
+      if (mysql->fields)
+      {
+	if (!(res= cli_use_result(mysql)))
+	  goto error;
+	mysql_free_result(res);
+      }
     }
     mysql->reconnect=reconnect;
   }
-#endif
 
 #ifndef TO_BE_DELETED
   if (mysql->options.rpl_probe && mysql_rpl_probe(mysql))
@@ -2586,9 +2526,30 @@ my_bool mysql_reconnect(MYSQL *mysql)
   tmp_mysql.reconnect= 1;
   tmp_mysql.free_me= mysql->free_me;
 
-  /* Move prepared statements (if any) over to the new mysql object */
-  tmp_mysql.stmts= mysql->stmts;
-  mysql->stmts= 0;
+  /*
+    For each stmt in mysql->stmts, move it to tmp_mysql if it is
+    in state MYSQL_STMT_INIT_DONE, otherwise close it.
+  */
+  {
+    LIST *element= mysql->stmts;
+    for (; element; element= element->next)
+    {
+      MYSQL_STMT *stmt= (MYSQL_STMT *) element->data;
+      if (stmt->state != MYSQL_STMT_INIT_DONE)
+      {
+        stmt->mysql= 0;
+        stmt->last_errno= CR_SERVER_LOST;
+        strmov(stmt->last_error, ER(CR_SERVER_LOST));
+        strmov(stmt->sqlstate, unknown_sqlstate);
+      }
+      else
+      {
+        tmp_mysql.stmts= list_add(tmp_mysql.stmts, &stmt->list);
+      }
+      /* No need to call list_delete for statement here */
+    }
+    mysql->stmts= NULL;
+  }
 
   /* Don't free options as these are now used in tmp_mysql */
   bzero((char*) &mysql->options,sizeof(mysql->options));
@@ -2678,46 +2639,6 @@ static void mysql_close_free(MYSQL *mysql)
 }
 
 
-/**
-  For use when the connection to the server has been lost (in which case 
-  the server has discarded all information about prepared statements
-  associated with the connection).
-
-  Mark all statements in mysql->stmts by setting stmt->mysql= 0 if the
-  statement has transitioned beyond the MYSQL_STMT_INIT_DONE state, and
-  unlink the statement from the mysql->stmts list.
-
-  The remaining pruned list of statements (if any) is kept in mysql->stmts.
-
-  @param mysql       pointer to the MYSQL object
-
-  @return none
-*/
-static void mysql_prune_stmt_list(MYSQL *mysql)
-{
-  LIST *element= mysql->stmts;
-  LIST *pruned_list= 0;
-
-  for (; element; element= element->next)
-  {
-    MYSQL_STMT *stmt= (MYSQL_STMT *) element->data;
-    if (stmt->state != MYSQL_STMT_INIT_DONE)
-    {
-      stmt->mysql= 0;
-      stmt->last_errno= CR_SERVER_LOST;
-      strmov(stmt->last_error, ER(CR_SERVER_LOST));
-      strmov(stmt->sqlstate, unknown_sqlstate);
-    }
-    else
-    {
-      pruned_list= list_add(pruned_list, element);
-    }
-  }
-
-  mysql->stmts= pruned_list;
-}
-
-
 /*
   Clear connection pointer of every statement: this is necessary
   to give error on attempt to use a prepared statement of closed
@@ -3262,7 +3183,7 @@ int STDCALL mysql_set_character_set(MYSQL *mysql, const char *cs_name)
     if (mysql_get_server_version(mysql) < 40100)
       return 0;
     sprintf(buff, "SET NAMES %s", cs_name);
-    if (!mysql_real_query(mysql, buff, (uint) strlen(buff)))
+    if (!mysql_real_query(mysql, buff, strlen(buff)))
     {
       mysql->charset= cs;
     }
