@@ -160,12 +160,12 @@ enum enum_mysql_timestamp_type
 str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
                 uint flags, int *was_cut)
 {
-  uint field_length, UNINIT_VAR(year_length), digits, i, number_of_fields;
+  uint field_length, year_length, digits, i, number_of_fields;
   uint date[MAX_DATE_PARTS], date_len[MAX_DATE_PARTS];
   uint add_hours= 0, start_loop;
   ulong not_zero_date, allow_space;
   my_bool is_internal_format;
-  const char *pos, *UNINIT_VAR(last_field_pos);
+  const char *pos, *last_field_pos;
   const char *end=str+length;
   const uchar *format_position;
   my_bool found_delimitier= 0, found_space= 0;
@@ -174,6 +174,8 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
   DBUG_PRINT("ENTER",("str: %.*s",length,str));
 
   LINT_INIT(field_length);
+  LINT_INIT(year_length);
+  LINT_INIT(last_field_pos);
 
   *was_cut= 0;
 
@@ -262,19 +264,8 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
   {
     const char *start= str;
     ulong tmp_value= (uint) (uchar) (*str++ - '0');
-
-    /*
-      Internal format means no delimiters; every field has a fixed
-      width. Otherwise, we scan until we find a delimiter and discard
-      leading zeroes -- except for the microsecond part, where leading
-      zeroes are significant, and where we never process more than six
-      digits.
-    */
-    my_bool     scan_until_delim= !is_internal_format &&
-                                  ((i != format_position[6]));
-
     while (str != end && my_isdigit(&my_charset_latin1,str[0]) &&
-           (scan_until_delim || --field_length))
+           (!is_internal_format || --field_length))
     {
       tmp_value=tmp_value*10 + (ulong) (uchar) (*str - '0');
       str++;
@@ -448,7 +439,9 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
     }
   }
 
-  DBUG_RETURN(l_time->time_type);
+  DBUG_RETURN(l_time->time_type=
+              (number_of_fields <= 3 ? MYSQL_TIMESTAMP_DATE :
+                                       MYSQL_TIMESTAMP_DATETIME));
 
 err:
   bzero((char*) l_time, sizeof(*l_time));
@@ -769,22 +762,19 @@ long calc_daynr(uint year,uint month,uint day)
 {
   long delsum;
   int temp;
-  int y= year;                                  /* may be < 0 temporarily */
   DBUG_ENTER("calc_daynr");
 
-  if (y == 0 && month == 0)
+  if (year == 0 && month == 0 && day == 0)
     DBUG_RETURN(0);				/* Skip errors */
-  /* Cast to int to be able to handle month == 0 */
-  delsum= (long) (365 * y + 31 *((int) month - 1) + (int) day);
+  delsum= (long) (365L * year+ 31*(month-1) +day);
   if (month <= 2)
-      y--;
+      year--;
   else
-    delsum-= (long) ((int) month * 4 + 23) / 10;
-  temp=(int) ((y/100+1)*3)/4;
+    delsum-= (long) (month*4+23)/10;
+  temp=(int) ((year/100+1)*3)/4;
   DBUG_PRINT("exit",("year: %d  month: %d  day: %d -> daynr: %ld",
-		     y+(month <= 2),month,day,delsum+y/4-temp));
-  DBUG_ASSERT(delsum+(int) y/4-temp > 0);
-  DBUG_RETURN(delsum+(int) y/4-temp);
+		     year+(month <= 2),month,day,delsum+year/4-temp));
+  DBUG_RETURN(delsum+(int) year/4-temp);
 } /* calc_daynr */
 
 
@@ -993,7 +983,7 @@ my_system_gmt_sec(const MYSQL_TIME *t_src, long *my_timezone,
     with unsigned time_t tmp+= shift*86400L might result in a number,
     larger then TIMESTAMP_MAX_VALUE, so another check will work.
   */
-  if (!IS_TIME_T_VALID_FOR_TIMESTAMP(tmp))
+  if ((tmp < TIMESTAMP_MIN_VALUE) || (tmp > TIMESTAMP_MAX_VALUE))
     tmp= 0;
 
   return (my_time_t) tmp;
@@ -1025,21 +1015,30 @@ void set_zero_time(MYSQL_TIME *tm, enum enum_mysql_timestamp_type time_type)
 int my_time_to_str(const MYSQL_TIME *l_time, char *to)
 {
   uint extra_hours= 0;
-  return sprintf(to, "%s%02u:%02u:%02u", (l_time->neg ? "-" : ""),
-                 extra_hours+ l_time->hour, l_time->minute, l_time->second);
+  return my_sprintf(to, (to, "%s%02u:%02u:%02u",
+                         (l_time->neg ? "-" : ""),
+                         extra_hours+ l_time->hour,
+                         l_time->minute,
+                         l_time->second));
 }
 
 int my_date_to_str(const MYSQL_TIME *l_time, char *to)
 {
-  return sprintf(to, "%04u-%02u-%02u",
-                 l_time->year, l_time->month, l_time->day);
+  return my_sprintf(to, (to, "%04u-%02u-%02u",
+                         l_time->year,
+                         l_time->month,
+                         l_time->day));
 }
 
 int my_datetime_to_str(const MYSQL_TIME *l_time, char *to)
 {
-  return sprintf(to, "%04u-%02u-%02u %02u:%02u:%02u",
-                 l_time->year, l_time->month, l_time->day,
-                 l_time->hour, l_time->minute, l_time->second);
+  return my_sprintf(to, (to, "%04u-%02u-%02u %02u:%02u:%02u",
+                         l_time->year,
+                         l_time->month,
+                         l_time->day,
+                         l_time->hour,
+                         l_time->minute,
+                         l_time->second));
 }
 
 
