@@ -278,9 +278,9 @@ static bool extract_date_time(DATE_TIME_FORMAT *format,
   int  strict_week_number_year= -1;
   int frac_part;
   bool usa_time= 0;
-  bool UNINIT_VAR(sunday_first_n_first_week_non_iso);
-  bool UNINIT_VAR(strict_week_number);
-  bool UNINIT_VAR(strict_week_number_year_type);
+  bool sunday_first_n_first_week_non_iso;
+  bool strict_week_number;
+  bool strict_week_number_year_type;
   const char *val_begin= val;
   const char *val_end= val + length;
   const char *ptr= format->format.str;
@@ -288,14 +288,19 @@ static bool extract_date_time(DATE_TIME_FORMAT *format,
   CHARSET_INFO *cs= &my_charset_bin;
   DBUG_ENTER("extract_date_time");
 
+  LINT_INIT(strict_week_number);
+  /* Remove valgrind varnings when using gcc 3.3 and -O1 */
+  PURIFY_OR_LINT_INIT(strict_week_number_year_type);
+  PURIFY_OR_LINT_INIT(sunday_first_n_first_week_non_iso);
+
   if (!sub_pattern_end)
     bzero((char*) l_time, sizeof(*l_time));
 
   for (; ptr != end && val != val_end; ptr++)
   {
     /* Skip pre-space between each argument */
-    if ((val+= cs->cset->scan(cs, val, val_end, MY_SEQ_SPACES)) >= val_end)
-      break;
+    while (val != val_end && my_isspace(cs, *val))
+      val++;
 
     if (*ptr == '%' && ptr+1 != end)
     {
@@ -386,7 +391,7 @@ static bool extract_date_time(DATE_TIME_FORMAT *format,
 	if (tmp - val > 6)
 	  tmp= (char*) val + 6;
 	l_time->second_part= (int) my_strtoll10(val, &tmp, &error);
-	frac_part= 6 - (int) (tmp - val);
+	frac_part= 6 - (tmp - val);
 	if (frac_part > 0)
 	  l_time->second_part*= (ulong) log_10_int[frac_part];
 	val= tmp;
@@ -441,7 +446,7 @@ static bool extract_date_time(DATE_TIME_FORMAT *format,
         strict_week_number= (*ptr=='V' || *ptr=='v');
 	tmp= (char*) val + min(val_len, 2);
 	if ((week_number= (int) my_strtoll10(val, &tmp, &error)) < 0 ||
-            (strict_week_number && !week_number) ||
+            strict_week_number && !week_number ||
             week_number > 53)
           goto err;
 	val= tmp;
@@ -537,10 +542,10 @@ static bool extract_date_time(DATE_TIME_FORMAT *format,
       %V,%v require %X,%x resprectively,
       %U,%u should be used with %Y and not %X or %x
     */
-    if ((strict_week_number &&
+    if (strict_week_number &&
         (strict_week_number_year < 0 ||
-         strict_week_number_year_type != sunday_first_n_first_week_non_iso)) ||
-        (!strict_week_number && strict_week_number_year >= 0))
+         strict_week_number_year_type != sunday_first_n_first_week_non_iso) ||
+        !strict_week_number && strict_week_number_year >= 0)
       goto err;
 
     /* Number of days since year 0 till 1st Jan of this year */
@@ -637,38 +642,38 @@ bool make_date_time(DATE_TIME_FORMAT *format, MYSQL_TIME *l_time,
         if (!l_time->month)
           return 1;
         str->append(locale->month_names->type_names[l_time->month-1],
-                    (uint) strlen(locale->month_names->type_names[l_time->month-1]),
+                    strlen(locale->month_names->type_names[l_time->month-1]),
                     system_charset_info);
         break;
       case 'b':
         if (!l_time->month)
           return 1;
         str->append(locale->ab_month_names->type_names[l_time->month-1],
-                    (uint) strlen(locale->ab_month_names->type_names[l_time->month-1]),
+                    strlen(locale->ab_month_names->type_names[l_time->month-1]),
                     system_charset_info);
         break;
       case 'W':
-        if (type == MYSQL_TIMESTAMP_TIME || !(l_time->month || l_time->year))
+        if (type == MYSQL_TIMESTAMP_TIME)
           return 1;
         weekday= calc_weekday(calc_daynr(l_time->year,l_time->month,
                               l_time->day),0);
         str->append(locale->day_names->type_names[weekday],
-                    (uint) strlen(locale->day_names->type_names[weekday]),
+                    strlen(locale->day_names->type_names[weekday]),
                     system_charset_info);
         break;
       case 'a':
-        if (type == MYSQL_TIMESTAMP_TIME || !(l_time->month || l_time->year))
+        if (type == MYSQL_TIMESTAMP_TIME)
           return 1;
         weekday=calc_weekday(calc_daynr(l_time->year,l_time->month,
                              l_time->day),0);
         str->append(locale->ab_day_names->type_names[weekday],
-                    (uint) strlen(locale->ab_day_names->type_names[weekday]),
+                    strlen(locale->ab_day_names->type_names[weekday]),
                     system_charset_info);
         break;
       case 'D':
 	if (type == MYSQL_TIMESTAMP_TIME)
 	  return 1;
-	length= (uint) (int10_to_str(l_time->day, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->day, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 1, '0');
 	if (l_time->day >= 10 &&  l_time->day <= 19)
 	  str->append(STRING_WITH_LEN("th"));
@@ -691,62 +696,62 @@ bool make_date_time(DATE_TIME_FORMAT *format, MYSQL_TIME *l_time,
 	}
 	break;
       case 'Y':
-	length= (uint) (int10_to_str(l_time->year, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->year, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 4, '0');
 	break;
       case 'y':
-	length= (uint) (int10_to_str(l_time->year%100, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->year%100, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'm':
-	length= (uint) (int10_to_str(l_time->month, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->month, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'c':
-	length= (uint) (int10_to_str(l_time->month, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->month, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 1, '0');
 	break;
       case 'd':
-	length= (uint) (int10_to_str(l_time->day, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->day, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'e':
-	length= (uint) (int10_to_str(l_time->day, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->day, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 1, '0');
 	break;
       case 'f':
-	length= (uint) (int10_to_str(l_time->second_part, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->second_part, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 6, '0');
 	break;
       case 'H':
-	length= (uint) (int10_to_str(l_time->hour, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->hour, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'h':
       case 'I':
 	hours_i= (l_time->hour%24 + 11)%12+1;
-	length= (uint) (int10_to_str(hours_i, intbuff, 10) - intbuff);
+	length= int10_to_str(hours_i, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'i':					/* minutes */
-	length= (uint) (int10_to_str(l_time->minute, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->minute, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'j':
 	if (type == MYSQL_TIMESTAMP_TIME)
 	  return 1;
-	length= (uint) (int10_to_str(calc_daynr(l_time->year,l_time->month,
+	length= int10_to_str(calc_daynr(l_time->year,l_time->month,
 					l_time->day) - 
-		     calc_daynr(l_time->year,1,1) + 1, intbuff, 10) - intbuff);
+		     calc_daynr(l_time->year,1,1) + 1, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 3, '0');
 	break;
       case 'k':
-	length= (uint) (int10_to_str(l_time->hour, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->hour, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 1, '0');
 	break;
       case 'l':
 	hours_i= (l_time->hour%24 + 11)%12+1;
-	length= (uint) (int10_to_str(hours_i, intbuff, 10) - intbuff);
+	length= int10_to_str(hours_i, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 1, '0');
 	break;
       case 'p':
@@ -754,21 +759,27 @@ bool make_date_time(DATE_TIME_FORMAT *format, MYSQL_TIME *l_time,
 	str->append(hours_i < 12 ? "AM" : "PM",2);
 	break;
       case 'r':
-	length= sprintf(intbuff, ((l_time->hour % 24) < 12) ?
-                        "%02d:%02d:%02d AM" : "%02d:%02d:%02d PM",
-		        (l_time->hour+11)%12+1,
-		        l_time->minute,
-		        l_time->second);
+	length= my_sprintf(intbuff, 
+		   (intbuff, 
+		    ((l_time->hour % 24) < 12) ?
+                    "%02d:%02d:%02d AM" : "%02d:%02d:%02d PM",
+		    (l_time->hour+11)%12+1,
+		    l_time->minute,
+		    l_time->second));
 	str->append(intbuff, length);
 	break;
       case 'S':
       case 's':
-	length= (uint) (int10_to_str(l_time->second, intbuff, 10) - intbuff);
+	length= int10_to_str(l_time->second, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
 	break;
       case 'T':
-	length= sprintf(intbuff,  "%02d:%02d:%02d",
-                        l_time->hour, l_time->minute, l_time->second);
+	length= my_sprintf(intbuff, 
+		   (intbuff, 
+		    "%02d:%02d:%02d", 
+		    l_time->hour, 
+		    l_time->minute,
+		    l_time->second));
 	str->append(intbuff, length);
 	break;
       case 'U':
@@ -777,11 +788,11 @@ bool make_date_time(DATE_TIME_FORMAT *format, MYSQL_TIME *l_time,
 	uint year;
 	if (type == MYSQL_TIMESTAMP_TIME)
 	  return 1;
-	length= (uint) (int10_to_str(calc_week(l_time,
+	length= int10_to_str(calc_week(l_time,
 				       (*ptr) == 'U' ?
 				       WEEK_FIRST_WEEKDAY : WEEK_MONDAY_FIRST,
 				       &year),
-			     intbuff, 10) - intbuff);
+			     intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
       }
       break;
@@ -791,12 +802,12 @@ bool make_date_time(DATE_TIME_FORMAT *format, MYSQL_TIME *l_time,
 	uint year;
 	if (type == MYSQL_TIMESTAMP_TIME)
 	  return 1;
-	length= (uint) (int10_to_str(calc_week(l_time,
+	length= int10_to_str(calc_week(l_time,
 				       ((*ptr) == 'V' ?
 					(WEEK_YEAR | WEEK_FIRST_WEEKDAY) :
 					(WEEK_YEAR | WEEK_MONDAY_FIRST)),
 				       &year),
-			     intbuff, 10) - intbuff);
+			     intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 2, '0');
       }
       break;
@@ -811,16 +822,16 @@ bool make_date_time(DATE_TIME_FORMAT *format, MYSQL_TIME *l_time,
 			  WEEK_YEAR | WEEK_FIRST_WEEKDAY :
 			  WEEK_YEAR | WEEK_MONDAY_FIRST),
 			 &year);
-	length= (uint) (int10_to_str(year, intbuff, 10) - intbuff);
+	length= int10_to_str(year, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 4, '0');
       }
       break;
       case 'w':
-	if (type == MYSQL_TIMESTAMP_TIME || !(l_time->month || l_time->year))
+	if (type == MYSQL_TIMESTAMP_TIME)
 	  return 1;
 	weekday=calc_weekday(calc_daynr(l_time->year,l_time->month,
 					l_time->day),1);
-	length= (uint) (int10_to_str(weekday, intbuff, 10) - intbuff);
+	length= int10_to_str(weekday, intbuff, 10) - intbuff;
 	str->append_with_prefill(intbuff, length, 1, '0');
 	break;
 
@@ -870,9 +881,9 @@ static bool get_interval_info(const char *str,uint length,CHARSET_INFO *cs,
       value= value*LL(10) + (longlong) (*str - '0');
     if (transform_msec && i == count - 1) // microseconds always last
     {
-      int msec_length= 6 - (int) (str - start);
+      long msec_length= 6 - (str - start);
       if (msec_length > 0)
-        value*= (long)log_10_int[msec_length];
+	value*= (long) log_10_int[msec_length];
     }
     values[i]= value;
     while (str != end && !my_isdigit(cs,*str))
@@ -949,9 +960,9 @@ enum_monotonicity_info Item_func_to_days::get_monotonicity_info() const
   if (args[0]->type() == Item::FIELD_ITEM)
   {
     if (args[0]->field_type() == MYSQL_TYPE_DATE)
-      return MONOTONIC_STRICT_INCREASING_NOT_NULL;
+      return MONOTONIC_STRICT_INCREASING;
     if (args[0]->field_type() == MYSQL_TYPE_DATETIME)
-      return MONOTONIC_INCREASING_NOT_NULL;
+      return MONOTONIC_INCREASING;
   }
   return NON_MONOTONIC;
 }
@@ -962,27 +973,12 @@ longlong Item_func_to_days::val_int_endpoint(bool left_endp, bool *incl_endp)
   DBUG_ASSERT(fixed == 1);
   MYSQL_TIME ltime;
   longlong res;
-  int dummy;                                /* unused */
-  if (get_arg0_date(&ltime, TIME_FUZZY_DATE))
+  if (get_arg0_date(&ltime, TIME_NO_ZERO_DATE))
   {
     /* got NULL, leave the incl_endp intact */
     return LONGLONG_MIN;
   }
   res=(longlong) calc_daynr(ltime.year,ltime.month,ltime.day);
-  /* Set to NULL if invalid date, but keep the value */
-  null_value= check_date(&ltime,
-                         (ltime.year || ltime.month || ltime.day),
-                         (TIME_NO_ZERO_IN_DATE | TIME_NO_ZERO_DATE),
-                         &dummy);
-  if (null_value)
-  {
-    /*
-      Even if the evaluation return NULL, the calc_daynr is useful for pruning
-    */
-    if (args[0]->field_type() != MYSQL_TYPE_DATE)
-      *incl_endp= TRUE;
-    return res;
-  }
   
   if (args[0]->field_type() == MYSQL_TYPE_DATE)
   {
@@ -995,19 +991,15 @@ longlong Item_func_to_days::val_int_endpoint(bool left_endp, bool *incl_endp)
     point to day bound ("strictly less" comparison stays intact):
 
       col < '2007-09-15 00:00:00'  -> TO_DAYS(col) <  TO_DAYS('2007-09-15')
-      col > '2007-09-15 23:59:59'  -> TO_DAYS(col) >  TO_DAYS('2007-09-15')
 
     which is different from the general case ("strictly less" changes to
     "less or equal"):
 
       col < '2007-09-15 12:34:56'  -> TO_DAYS(col) <= TO_DAYS('2007-09-15')
   */
-  if ((!left_endp && !(ltime.hour || ltime.minute || ltime.second ||
-                       ltime.second_part)) ||
-       (left_endp && ltime.hour == 23 && ltime.minute == 59 &&
-        ltime.second == 59))
-    /* do nothing */
-    ;
+  if (!left_endp && !(ltime.hour || ltime.minute || ltime.second ||
+                      ltime.second_part))
+    ; /* do nothing */
   else
     *incl_endp= TRUE;
   return res;
@@ -1041,25 +1033,12 @@ longlong Item_func_month::val_int()
 }
 
 
-void Item_func_monthname::fix_length_and_dec()
-{
-  THD* thd= current_thd;
-  CHARSET_INFO *cs= thd->variables.collation_connection;
-  uint32 repertoire= my_charset_repertoire(cs);
-  locale= thd->variables.lc_time_names;  
-  collation.set(cs, DERIVATION_COERCIBLE, repertoire);
-  decimals=0;
-  max_length= locale->max_month_name_length * collation.collation->mbmaxlen;
-  maybe_null=1; 
-}
-
-
 String* Item_func_monthname::val_str(String* str)
 {
   DBUG_ASSERT(fixed == 1);
   const char *month_name;
-  uint month= (uint) val_int();
-  uint err;
+  uint   month= (uint) val_int();
+  THD *thd= current_thd;
 
   if (null_value || !month)
   {
@@ -1067,9 +1046,8 @@ String* Item_func_monthname::val_str(String* str)
     return (String*) 0;
   }
   null_value=0;
-  month_name= locale->month_names->type_names[month-1];
-  str->copy(month_name, (uint) strlen(month_name), &my_charset_utf8_bin,
-	    collation.collation, &err);
+  month_name= thd->variables.lc_time_names->month_names->type_names[month-1];
+  str->set(month_name, strlen(month_name), system_charset_info);
   return str;
 }
 
@@ -1194,32 +1172,19 @@ longlong Item_func_weekday::val_int()
                                  odbc_type) + test(odbc_type);
 }
 
-void Item_func_dayname::fix_length_and_dec()
-{
-  THD* thd= current_thd;
-  CHARSET_INFO *cs= thd->variables.collation_connection;
-  uint32 repertoire= my_charset_repertoire(cs);
-  locale= thd->variables.lc_time_names;  
-  collation.set(cs, DERIVATION_COERCIBLE, repertoire);
-  decimals=0;
-  max_length= locale->max_day_name_length * collation.collation->mbmaxlen;
-  maybe_null=1; 
-}
-
 
 String* Item_func_dayname::val_str(String* str)
 {
   DBUG_ASSERT(fixed == 1);
   uint weekday=(uint) val_int();		// Always Item_func_daynr()
   const char *day_name;
-  uint err;
+  THD *thd= current_thd;
 
   if (null_value)
     return (String*) 0;
   
-  day_name= locale->day_names->type_names[weekday];
-  str->copy(day_name, (uint) strlen(day_name), &my_charset_utf8_bin,
-	    collation.collation, &err);
+  day_name= thd->variables.lc_time_names->day_names->type_names[weekday];
+  str->set(day_name, strlen(day_name), system_charset_info);
   return str;
 }
 
@@ -1338,11 +1303,15 @@ bool get_interval_value(Item *args,interval_type int_type,
 			       String *str_value, INTERVAL *interval)
 {
   ulonglong array[5];
-  longlong UNINIT_VAR(value);
-  const char *UNINIT_VAR(str);
-  size_t UNINIT_VAR(length);
+  longlong value;
+  const char *str;
+  size_t length;
   CHARSET_INFO *cs=str_value->charset();
 
+  LINT_INIT(value);
+  LINT_INIT(str);
+  LINT_INIT(length);
+
   bzero((char*) interval,sizeof(*interval));
   if ((int) int_type <= INTERVAL_MICROSECOND)
   {
@@ -1519,11 +1488,6 @@ bool Item_func_from_days::get_date(MYSQL_TIME *ltime, uint fuzzy_date)
     return 1;
   bzero(ltime, sizeof(MYSQL_TIME));
   get_date_from_daynr((long) value, &ltime->year, &ltime->month, &ltime->day);
-
-  if ((null_value= (fuzzy_date & TIME_NO_ZERO_DATE) &&
-       (ltime->year == 0 || ltime->month == 0 || ltime->day == 0)))
-    return TRUE;
-
   ltime->time_type= MYSQL_TIMESTAMP_DATE;
   return 0;
 }
@@ -2275,6 +2239,8 @@ void Item_extract::print(String *str, enum_query_type query_type)
 
 void Item_extract::fix_length_and_dec()
 {
+  value.alloc(32);				// alloc buffer
+
   maybe_null=1;					// If wrong date
   switch (int_type) {
   case INTERVAL_YEAR:		max_length=4; date_value=1; break;
@@ -2317,8 +2283,6 @@ longlong Item_extract::val_int()
   }
   else
   {
-    char buf[40];
-    String value(buf, sizeof(buf), &my_charset_bin);;
     String *res= args[0]->val_str(&value);
     if (!res || str_to_time_with_warn(res->ptr(), res->length(), &ltime))
     {
@@ -2449,19 +2413,6 @@ String *Item_char_typecast::val_str(String *str)
   String *res;
   uint32 length;
 
-  if (cast_length >= 0 &&
-      ((unsigned) cast_length) > current_thd->variables.max_allowed_packet)
-  {
-    push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-			ER_WARN_ALLOWED_PACKET_OVERFLOWED,
-			ER(ER_WARN_ALLOWED_PACKET_OVERFLOWED),
-			cast_cs == &my_charset_bin ?
-                        "cast_as_binary" : func_name(),
-                        current_thd->variables.max_allowed_packet);
-    null_value= 1;
-    return 0;
-  }
-
   if (!charset_conversion)
   {
     if (!(res= args[0]->val_str(str)))
@@ -2474,14 +2425,14 @@ String *Item_char_typecast::val_str(String *str)
   {
     // Convert character set if differ
     uint dummy_errors;
-    if (!(res= args[0]->val_str(str)) ||
-        tmp_value.copy(res->ptr(), res->length(), from_cs,
-                       cast_cs, &dummy_errors))
+    if (!(res= args[0]->val_str(&tmp_value)) ||
+        str->copy(res->ptr(), res->length(), from_cs,
+        cast_cs, &dummy_errors))
     {
       null_value= 1;
       return 0;
     }
-    res= &tmp_value;
+    res= str;
   }
 
   res->set_charset(cast_cs);
@@ -2515,9 +2466,9 @@ String *Item_char_typecast::val_str(String *str)
     {
       if (res->alloced_length() < (uint) cast_length)
       {
-        str_value.alloc(cast_length);
-        str_value.copy(*res);
-        res= &str_value;
+        str->alloc(cast_length);
+        str->copy(*res);
+        res= str;
       }
       bzero((char*) res->ptr() + res->length(),
             (uint) cast_length - res->length());
@@ -2553,8 +2504,6 @@ void Item_char_typecast::fix_length_and_dec()
        and thus avoid unnecessary character set conversion.
      - If the argument is not a number, then from_cs is set to
        the argument's charset.
-
-       Note (TODO): we could use repertoire technique here.
   */
   from_cs= (args[0]->result_type() == INT_RESULT || 
             args[0]->result_type() == DECIMAL_RESULT ||
@@ -2562,13 +2511,12 @@ void Item_char_typecast::fix_length_and_dec()
            (cast_cs->mbminlen == 1 ? cast_cs : &my_charset_latin1) :
            args[0]->collation.collation;
   charset_conversion= (cast_cs->mbmaxlen > 1) ||
-                      (!my_charset_same(from_cs, cast_cs) &&
-                       from_cs != &my_charset_bin &&
-                       cast_cs != &my_charset_bin);
+                      !my_charset_same(from_cs, cast_cs) &&
+                      from_cs != &my_charset_bin &&
+                      cast_cs != &my_charset_bin;
   collation.set(cast_cs, DERIVATION_IMPLICIT);
-  char_length= (cast_length >= 0) ? cast_length :
-                args[0]->max_length /
-                (cast_cs == &my_charset_bin ? 1 : args[0]->collation.collation->mbmaxlen);
+  char_length= (cast_length >= 0) ? cast_length : 
+	       args[0]->max_length/from_cs->mbmaxlen;
   max_length= char_length * cast_cs->mbmaxlen;
 }
 
@@ -2702,7 +2650,7 @@ String *Item_func_makedate::val_str(String *str)
   long days;
 
   if (args[0]->null_value || args[1]->null_value ||
-      year < 0 || year > 9999 || daynr <= 0)
+      year < 0 || daynr <= 0)
     goto err;
 
   if (year < 100)
@@ -2745,7 +2693,7 @@ longlong Item_func_makedate::val_int()
   long days;
 
   if (args[0]->null_value || args[1]->null_value ||
-      year < 0 || year > 9999 || daynr <= 0)
+      year < 0 || daynr <= 0)
     goto err;
 
   if (year < 100)
@@ -2997,12 +2945,12 @@ String *Item_func_maketime::val_str(String *str)
     char buf[28];
     char *ptr= longlong10_to_str(hour, buf, args[0]->unsigned_flag ? 10 : -10);
     int len = (int)(ptr - buf) +
-      sprintf(ptr, ":%02u:%02u", (uint) minute, (uint) second);
+      my_sprintf(ptr, (ptr, ":%02u:%02u", (uint)minute, (uint)second));
     make_truncated_value_warning(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
                                  buf, len, MYSQL_TIMESTAMP_TIME,
                                  NullS);
   }
-
+  
   if (make_time_with_warn((DATE_TIME_FORMAT *) 0, &ltime, str))
   {
     null_value= 1;
@@ -3198,14 +3146,14 @@ String *Item_func_get_format::val_str(String *str)
        format++)
   {
     uint format_name_len;
-    format_name_len= (uint) strlen(format_name);
+    format_name_len= strlen(format_name);
     if (val_len == format_name_len &&
 	!my_strnncoll(&my_charset_latin1, 
 		      (const uchar *) val->ptr(), val_len, 
 		      (const uchar *) format_name, val_len))
     {
       const char *format_str= get_date_time_format_str(format, type);
-      str->set(format_str, (uint) strlen(format_str), &my_charset_bin);
+      str->set(format_str, strlen(format_str), &my_charset_bin);
       return str;
     }
   }
@@ -3311,7 +3259,6 @@ void Item_func_str_to_date::fix_length_and_dec()
 {
   maybe_null= 1;
   decimals=0;
-  cached_format_type= DATE_TIME;
   cached_field_type= MYSQL_TYPE_DATETIME;
   max_length= MAX_DATETIME_FULL_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
   cached_timestamp_type= MYSQL_TIMESTAMP_NONE;
