@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -70,10 +70,6 @@ public:
   enum_monotonicity_info get_monotonicity_info() const;
   longlong val_int_endpoint(bool left_endp, bool *incl_endp);
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
@@ -90,10 +86,6 @@ public:
     maybe_null=1; 
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
@@ -106,11 +98,8 @@ public:
   { DBUG_ASSERT(fixed == 1); return (double) Item_func_month::val_int(); }
   String *val_str(String *str) 
   {
-    longlong nr= val_int();
-    if (null_value)
-      return 0;
-    str->set(nr, &my_charset_bin);
-    return str;
+    str->set(val_int(), &my_charset_bin);
+    return null_value ? 0 : str;
   }
   const char *func_name() const { return "month"; }
   enum Item_result result_type () const { return INT_RESULT; }
@@ -122,22 +111,23 @@ public:
     maybe_null=1; 
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
 class Item_func_monthname :public Item_func_month
 {
-  MY_LOCALE *locale;
 public:
   Item_func_monthname(Item *a) :Item_func_month(a) {}
   const char *func_name() const { return "monthname"; }
   String *val_str(String *str);
   enum Item_result result_type () const { return STRING_RESULT; }
-  void fix_length_and_dec();
+  void fix_length_and_dec() 
+  {
+    collation.set(&my_charset_bin);
+    decimals=0;
+    max_length=10*my_charset_bin.mbmaxlen;
+    maybe_null=1; 
+  }
   bool check_partition_func_processor(uchar *int_arg) {return TRUE;}
 };
 
@@ -155,10 +145,6 @@ public:
     maybe_null=1; 
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
@@ -175,10 +161,6 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_time_args();
-  }
 };
 
 
@@ -195,10 +177,6 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_time_args();
-  }
 };
 
 
@@ -215,10 +193,6 @@ public:
      maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
@@ -235,10 +209,6 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_time_args();
-  }
 };
 
 
@@ -269,10 +239,6 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
@@ -291,10 +257,6 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 
@@ -325,21 +287,22 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_date_args();
-  }
 };
 
 class Item_func_dayname :public Item_func_weekday
 {
-  MY_LOCALE *locale;
  public:
   Item_func_dayname(Item *a) :Item_func_weekday(a,0) {}
   const char *func_name() const { return "dayname"; }
   String *val_str(String *str);
   enum Item_result result_type () const { return STRING_RESULT; }
-  void fix_length_and_dec();
+  void fix_length_and_dec() 
+  { 
+    collation.set(&my_charset_bin);
+    decimals=0; 
+    max_length=9*MY_CHARSET_BIN_MB_MAXLEN;
+    maybe_null=1; 
+  }
   bool check_partition_func_processor(uchar *int_arg) {return TRUE;}
 };
 
@@ -352,16 +315,6 @@ public:
   Item_func_unix_timestamp(Item *a) :Item_int_func(a) {}
   longlong val_int();
   const char *func_name() const { return "unix_timestamp"; }
-  bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  /*
-    UNIX_TIMESTAMP() depends on the current timezone
-    (and thus may not be used as a partitioning function)
-    when its argument is NOT of the TIMESTAMP type.
-  */
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_timestamp_args();
-  }
   void fix_length_and_dec()
   {
     decimals=0;
@@ -378,15 +331,10 @@ public:
   const char *func_name() const { return "time_to_sec"; }
   void fix_length_and_dec()
   {
-    maybe_null= TRUE;
     decimals=0;
     max_length=10*MY_CHARSET_BIN_MB_MAXLEN;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_time_args();
-  }
 };
 
 
@@ -481,8 +429,6 @@ public:
   {
     return save_time_in_field(field);
   }
-  longlong val_int() { return val_int_from_decimal(); }
-  bool result_as_longlong() { return TRUE; }
 };
 
 
@@ -640,10 +586,6 @@ public:
   const char *func_name() const { return "from_days"; }
   bool get_date(MYSQL_TIME *res, uint fuzzy_date);
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return has_date_args() || has_time_args();
-  }
 };
 
 
@@ -758,6 +700,7 @@ public:
 
 class Item_extract :public Item_int_func
 {
+  String value;
   bool date_value;
  public:
   const interval_type int_type; // keep it public
@@ -770,42 +713,6 @@ class Item_extract :public Item_int_func
   bool eq(const Item *item, bool binary_cmp) const;
   virtual void print(String *str, enum_query_type query_type);
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    switch (int_type) {
-    case INTERVAL_YEAR:
-    case INTERVAL_YEAR_MONTH:
-    case INTERVAL_QUARTER:
-    case INTERVAL_MONTH:
-    /* case INTERVAL_WEEK: Not allowed as partitioning function, bug#57071 */
-    case INTERVAL_DAY:
-      return !has_date_args();
-    case INTERVAL_DAY_HOUR:
-    case INTERVAL_DAY_MINUTE:
-    case INTERVAL_DAY_SECOND:
-    case INTERVAL_DAY_MICROSECOND:
-      return !has_datetime_args();
-    case INTERVAL_HOUR:
-    case INTERVAL_HOUR_MINUTE:
-    case INTERVAL_HOUR_SECOND:
-    case INTERVAL_MINUTE:
-    case INTERVAL_MINUTE_SECOND:
-    case INTERVAL_SECOND:
-    case INTERVAL_MICROSECOND:
-    case INTERVAL_HOUR_MICROSECOND:
-    case INTERVAL_MINUTE_MICROSECOND:
-    case INTERVAL_SECOND_MICROSECOND:
-      return !has_time_args();
-    default:
-      /*
-        INTERVAL_LAST is only an end marker,
-        INTERVAL_WEEK depends on default_week_format which is a session
-        variable and cannot be used for partitioning. See bug#57071.
-      */
-      break;
-    }
-    return true;
-  }
 };
 
 
@@ -972,8 +879,6 @@ public:
   { 
     decimals=0;
     max_length=MAX_DATE_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
-    /* It returns NULL when the second argument is less or equal to 0 */
-    maybe_null= 1;
   }
   longlong val_int();
 };
@@ -1056,10 +961,6 @@ public:
     maybe_null=1;
   }
   bool check_partition_func_processor(uchar *int_arg) {return FALSE;}
-  bool check_valid_arguments_processor(uchar *int_arg)
-  {
-    return !has_time_args();
-  }
 };
 
 
