@@ -20,27 +20,27 @@
 #define PARSE_FILE_TIMESTAMPLENGTH 19
 
 enum file_opt_type {
-  FILE_OPTIONS_STRING,		/* String (LEX_STRING) */
-  FILE_OPTIONS_ESTRING,		/* Escaped string (LEX_STRING) */
-  FILE_OPTIONS_ULONGLONG,	/* ulonglong parameter (ulonglong) */
-  FILE_OPTIONS_REV,		/* Revision version number (ulonglong) */
-  FILE_OPTIONS_TIMESTAMP,	/* timestamp (LEX_STRING have to be
+  FILE_OPTIONS_STRING,		/**< String (LEX_STRING) */
+  FILE_OPTIONS_ESTRING,		/**< Escaped string (LEX_STRING) */
+  FILE_OPTIONS_ULONGLONG,	/**< ulonglong parameter (ulonglong) */
+  FILE_OPTIONS_REV,		/**< Revision version number (ulonglong) */
+  FILE_OPTIONS_TIMESTAMP,	/**< timestamp (LEX_STRING have to be
 				   allocated with length 20 (19+1) */
-  FILE_OPTIONS_STRLIST,         /* list of escaped strings
+  FILE_OPTIONS_STRLIST,         /**< list of escaped strings
                                    (List<LEX_STRING>) */
-  FILE_OPTIONS_ULLLIST          /* list of ulonglong values
+  FILE_OPTIONS_ULLLIST          /**< list of ulonglong values
                                    (List<ulonglong>) */
 };
 
 struct File_option
 {
-  LEX_STRING name;		/* Name of the option */
-  int offset;			/* offset to base address of value */
-  file_opt_type type;		/* Option type */
+  LEX_STRING name;		/**< Name of the option */
+  int offset;			/**< offset to base address of value */
+  file_opt_type type;		/**< Option type */
 };
 
 
-/*
+/**
   This hook used to catch no longer supported keys and process them for
   backward compatibility.
 */
@@ -50,25 +50,25 @@ class Unknown_key_hook
 public:
   Unknown_key_hook() {}                       /* Remove gcc warning */
   virtual ~Unknown_key_hook() {}              /* Remove gcc warning */
-  virtual bool process_unknown_string(char *&unknown_key, gptr base,
+  virtual bool process_unknown_string(char *&unknown_key, uchar* base,
                                       MEM_ROOT *mem_root, char *end)= 0;
 };
 
 
-/* Dummy hook for parsers which do not need hook for unknown keys */
+/** Dummy hook for parsers which do not need hook for unknown keys. */
 
 class File_parser_dummy_hook: public Unknown_key_hook
 {
 public:
   File_parser_dummy_hook() {}                 /* Remove gcc warning */
-  virtual bool process_unknown_string(char *&unknown_key, gptr base,
+  virtual bool process_unknown_string(char *&unknown_key, uchar* base,
                                       MEM_ROOT *mem_root, char *end);
 };
 
 extern File_parser_dummy_hook file_parser_dummy_hook;
 
 bool get_file_options_ulllist(char *&ptr, char *end, char *line,
-                              gptr base, File_option *parameter,
+                              uchar* base, File_option *parameter,
                               MEM_ROOT *mem_root);
 
 char *
@@ -81,7 +81,7 @@ File_parser *sql_parse_prepare(const LEX_STRING *file_name,
 my_bool
 sql_create_definition_file(const LEX_STRING *dir, const  LEX_STRING *file_name,
 			   const LEX_STRING *type,
-			   gptr base, File_option *parameters, uint versions);
+			   uchar* base, File_option *parameters, uint versions);
 my_bool rename_in_schema_file(THD *thd,
                               const char *schema, const char *old_name,
                               const char *new_name, ulonglong revision,
@@ -98,7 +98,7 @@ public:
 
   my_bool ok() { return content_ok; }
   LEX_STRING *type() { return &file_type; }
-  my_bool parse(gptr base, MEM_ROOT *mem_root,
+  my_bool parse(uchar* base, MEM_ROOT *mem_root,
 		struct File_option *parameters, uint required,
                 Unknown_key_hook *hook);
 
@@ -106,21 +106,4 @@ public:
 					MEM_ROOT *mem_root,
 					bool bad_format_errors);
 };
-
-
-/*
-  Custom version of standard offsetof() macro which can be used to get
-  offsets of members in class for non-POD types (according to the current
-  version of C++ standard offsetof() macro can't be used in such cases and
-  attempt to do so causes warnings to be emitted, OTOH in many cases it is
-  still OK to assume that all instances of the class has the same offsets
-  for the same members).
-
-  This is temporary solution which should be removed once File_parser class
-  and related routines are refactored.
-*/
-
-#define my_offsetof(TYPE, MEMBER) \
-        ((size_t)((char *)&(((TYPE *)0x10)->MEMBER) - (char*)0x10))
-
 #endif /* _PARSE_FILE_H_ */
