@@ -483,8 +483,10 @@ static bool check_engine_condition(partition_element *p_elem,
   {
     DBUG_RETURN(TRUE);
   }
-
-  DBUG_RETURN(FALSE);
+  else
+  {
+    DBUG_RETURN(FALSE);
+  }
 }
 
 
@@ -865,7 +867,6 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
 
     if (part_type != HASH_PARTITION || !list_of_part_fields)
     {
-      DBUG_ASSERT(part_expr);
       err= part_expr->walk(&Item::check_partition_func_processor, 0,
                            NULL);
       if (!err && is_sub_partitioned() && !list_of_subpart_fields)
@@ -955,13 +956,11 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
 #endif
       {
         if (part_elem->data_file_name)
-          push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                              WARN_OPTION_IGNORED, ER(WARN_OPTION_IGNORED),
-                              "DATA DIRECTORY");
+          push_warning(thd, MYSQL_ERROR::WARN_LEVEL_WARN, 0,
+                       "DATA DIRECTORY option ignored");
         if (part_elem->index_file_name)
-          push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                              WARN_OPTION_IGNORED, ER(WARN_OPTION_IGNORED),
-                              "INDEX DIRECTORY");
+          push_warning(thd, MYSQL_ERROR::WARN_LEVEL_WARN, 0,
+                       "INDEX DIRECTORY option ignored");
         part_elem->data_file_name= part_elem->index_file_name= NULL;
       }
       if (!is_sub_partitioned())
@@ -972,7 +971,7 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
           part_elem->engine_type= default_engine_type;
         }
         if (check_table_name(part_elem->partition_name,
-                             strlen(part_elem->partition_name), FALSE))
+                             strlen(part_elem->partition_name)))
         {
           my_error(ER_WRONG_PARTITION_NAME, MYF(0));
           goto end;
@@ -990,7 +989,7 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
         {
           sub_elem= sub_it++;
           if (check_table_name(sub_elem->partition_name,
-                               strlen(sub_elem->partition_name), FALSE))
+                               strlen(sub_elem->partition_name)))
           {
             my_error(ER_WRONG_PARTITION_NAME, MYF(0));
             goto end;
