@@ -559,12 +559,7 @@ HOSTS";
 	goto err;
       }
       si->server_id = log_server_id;
-      if (my_hash_insert(&slave_list, (uchar*)si))
-      {
-        error= "the slave is out of memory";
-        pthread_mutex_unlock(&LOCK_slave_list);
-        goto err;
-      }
+      my_hash_insert(&slave_list, (uchar*)si);
     }
     strmake(si->host, row[1], sizeof(si->host)-1);
     si->port = atoi(row[port_ind]);
@@ -643,24 +638,12 @@ err:
   if (recovery_captain)
     mysql_close(recovery_captain);
   delete thd;
-
-  DBUG_LEAVE;                                   // Must match DBUG_ENTER()
   my_thread_end();
   pthread_exit(0);
-  return 0;                                     // Avoid compiler warnings
+  DBUG_RETURN(0);
 }
 #endif
 
-
-/**
-  Execute a SHOW SLAVE HOSTS statement.
-
-  @param thd Pointer to THD object for the client thread executing the
-  statement.
-
-  @retval FALSE success
-  @retval TRUE failure
-*/
 bool show_slave_hosts(THD* thd)
 {
   List<Item> field_list;
