@@ -179,7 +179,6 @@ exit:
     {
       if (thd->is_error() &&
           (thd->main_da.sql_errno() == ER_BAD_FIELD_ERROR ||
-          thd->main_da.sql_errno() == ER_FUNC_INEXISTENT_NAME_COLLISION ||
           thd->main_da.sql_errno() == ER_SP_DOES_NOT_EXIST))
       {
         thd->clear_error();
@@ -281,13 +280,13 @@ bool mysql_derived_filling(THD *thd, LEX *lex, TABLE_LIST *orig_table_list)
 
       lex->current_select= first_select;
       res= mysql_select(thd, &first_select->ref_pointer_array,
-			first_select->table_list.first,
+			(TABLE_LIST*) first_select->table_list.first,
 			first_select->with_wild,
 			first_select->item_list, first_select->where,
 			(first_select->order_list.elements+
 			 first_select->group_list.elements),
-			first_select->order_list.first,
-			first_select->group_list.first,
+			(ORDER *) first_select->order_list.first,
+			(ORDER *) first_select->group_list.first,
 			first_select->having, (ORDER*) NULL,
 			(first_select->options | thd->options |
 			 SELECT_NO_UNLOCK),
