@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
 
 /*
   Functions to create a unireg form-file from a FIELD and a fieldname-fieldinfo
@@ -36,7 +37,8 @@ static bool pack_header(uchar *forminfo,enum legacy_db_type table_type,
 			List<Create_field> &create_fields,
 			uint info_length, uint screens, uint table_options,
 			ulong data_offset, handler *file);
-static uint get_interval_id(uint *,List<Create_field> &, Create_field *);
+static uint get_interval_id(uint *int_count,List<Create_field> &create_fields,
+			    Create_field *last_field);
 static bool pack_fields(File file, List<Create_field> &create_fields,
                         ulong data_offset);
 static bool make_empty_rec(THD *thd, int file, enum legacy_db_type table_type,
@@ -90,8 +92,8 @@ handle_error(uint sql_errno,
     db_file		Handler to use. May be zero, in which case we use
 			create_info->db_type
   RETURN
-    false  ok
-    true   error
+    0  ok
+    1  error
 */
 
 bool mysql_create_frm(THD *thd, const char *file_name,
@@ -107,6 +109,7 @@ bool mysql_create_frm(THD *thd, const char *file_name,
   File file;
   ulong filepos, data_offset;
   uchar fileinfo[64],forminfo[288],*keybuff;
+  TYPELIB formnames;
   uchar *screen_buff;
   char buff[128];
 #ifdef WITH_PARTITION_STORAGE_ENGINE
@@ -117,7 +120,7 @@ bool mysql_create_frm(THD *thd, const char *file_name,
   DBUG_ENTER("mysql_create_frm");
 
   DBUG_ASSERT(*fn_rext((char*)file_name)); // Check .frm extension
-
+  formnames.type_names=0;
   if (!(screen_buff=pack_screens(create_fields,&info_length,&screens,0)))
     DBUG_RETURN(1);
   DBUG_ASSERT(db_file != NULL);
@@ -192,15 +195,8 @@ bool mysql_create_frm(THD *thd, const char *file_name,
   key_buff_length= uint4korr(fileinfo+47);
   keybuff=(uchar*) my_malloc(key_buff_length, MYF(0));
   key_info_length= pack_keys(keybuff, keys, key_info, data_offset);
-
-  /*
-    Ensure that there are no forms in this newly created form file.
-    Even if the form file exists, create_frm must truncate it to
-    ensure one form per form file.
-  */
-  DBUG_ASSERT(uint2korr(fileinfo+8) == 0);
-
-  if (!(filepos= make_new_entry(file, fileinfo, NULL, "")))
+  VOID(get_form_pos(file,fileinfo,&formnames));
+  if (!(filepos=make_new_entry(file,fileinfo,&formnames,"")))
     goto err;
   maxlength=(uint) next_io_size((ulong) (uint2korr(forminfo)+1000));
   int2store(forminfo+2,maxlength);
@@ -209,47 +205,36 @@ bool mysql_create_frm(THD *thd, const char *file_name,
 			     (create_info->min_rows == 1) && (keys == 0));
   int2store(fileinfo+28,key_info_length);
 
-  /*
-    This gives us the byte-position of the character at
-    (character-position, not byte-position) TABLE_COMMENT_MAXLEN.
-    The trick here is that character-positions start at 0, so the last
-    character in a maximum-allowed length string would be at char-pos
-    MAXLEN-1; charpos MAXLEN will be the position of the terminator.
-    Consequently, bytepos(charpos(MAXLEN)) should be equal to
-    comment[length] (which should also be the terminator, or at least
-    the first byte after the payload in the strict sense). If this is
-    not so (bytepos(charpos(MAXLEN)) comes /before/ the end of the
-    string), the string is too long.
-
-    For additional credit, realise that UTF-8 has 1-3 bytes before 6.0,
-    and 1-4 bytes in 6.0 (6.0 also has UTF-32). This means that the
-    inlined COMMENT supposedly does not exceed 60 character plus
-    terminator, vulgo, 181 bytes.
-  */
-
   tmp_len= system_charset_info->cset->charpos(system_charset_info,
                                               create_info->comment.str,
                                               create_info->comment.str +
                                               create_info->comment.length, 60);
   if (tmp_len < create_info->comment.length)
   {
+    (void) my_snprintf(buff, sizeof(buff), "Too long comment for table '%s'",
+                       table);
     if ((thd->variables.sql_mode &
          (MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES)))
     {
-      my_error(ER_TOO_LONG_TABLE_COMMENT, MYF(0), table,
-               static_cast<ulong>(tmp_len));
+      my_message(ER_UNKNOWN_ERROR, buff, MYF(0));
       goto err;
     }
     push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                        ER_TOO_LONG_TABLE_COMMENT,
-                        ER(ER_TOO_LONG_TABLE_COMMENT),
-                        table, static_cast<ulong>(tmp_len));
+                        ER_UNKNOWN_ERROR, ER(ER_UNKNOWN_ERROR), buff);
     create_info->comment.length= tmp_len;
   }
 
   strmake((char*) forminfo+47, create_info->comment.str ?
           create_info->comment.str : "", create_info->comment.length);
   forminfo[46]=(uchar) create_info->comment.length;
+#ifdef EXTRA_DEBUG
+  /*
+    EXTRA_DEBUG causes strmake() to initialize its buffer behind the
+    payload with a magic value to detect wrong buffer-sizes. We
+    explicitly zero that segment again.
+  */
+  memset((char*) forminfo+47 + forminfo[46], 0, 61 - forminfo[46]);
+#endif
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   if (part_info)
   {
@@ -418,10 +403,10 @@ int rea_create_table(THD *thd, const char *path,
   DBUG_ASSERT(*fn_rext(frm_name));
   if (thd->variables.keep_files_on_create)
     create_info->options|= HA_CREATE_KEEP_FILES;
-  if (!create_info->frm_only &&
-      (file->ha_create_handler_files(path, NULL, CHF_CREATE_FLAG,
-                                     create_info) ||
-       ha_create_table(thd, path, db, table_name, create_info, 0)))
+  if (file->ha_create_handler_files(path, NULL, CHF_CREATE_FLAG, create_info))
+    goto err_handler;
+  if (!create_info->frm_only && ha_create_table(thd, path, db, table_name,
+                                                create_info,0))
     goto err_handler;
   DBUG_RETURN(0);
 
@@ -618,17 +603,17 @@ static bool pack_header(uchar *forminfo, enum legacy_db_type table_type,
                                                      255);
     if (tmp_len < field->comment.length)
     {
+      char buff[128];
+      (void) my_snprintf(buff,sizeof(buff), "Too long comment for field '%s'",
+                         field->field_name);
       if ((current_thd->variables.sql_mode &
 	   (MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES)))
       {
-        my_error(ER_TOO_LONG_FIELD_COMMENT, MYF(0), field->field_name,
-                 static_cast<ulong>(tmp_len));
+        my_message(ER_UNKNOWN_ERROR, buff, MYF(0));
 	DBUG_RETURN(1);
       }
       push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
-                          ER_TOO_LONG_FIELD_COMMENT,
-                          ER(ER_TOO_LONG_FIELD_COMMENT),
-                          field->field_name, static_cast<ulong>(tmp_len));
+                          ER_UNKNOWN_ERROR, ER(ER_UNKNOWN_ERROR), buff);
       field->comment.length= tmp_len;
     }
 
@@ -712,7 +697,7 @@ static bool pack_header(uchar *forminfo, enum legacy_db_type table_type,
 
   if (reclength > (ulong) file->max_record_length())
   {
-    my_error(ER_TOO_BIG_ROWSIZE, MYF(0), static_cast<long>(file->max_record_length()));
+    my_error(ER_TOO_BIG_ROWSIZE, MYF(0), (uint) file->max_record_length());
     DBUG_RETURN(1);
   }
   /* Hack to avoid bugs with small static rows in MySQL */
