@@ -91,7 +91,7 @@ const char **ha_heap::bas_ext() const
 int ha_heap::open(const char *name, int mode, uint test_if_locked)
 {
   if ((test_if_locked & HA_OPEN_INTERNAL_TABLE) ||
-      (!(file= heap_open(name, mode)) && my_errno == ENOENT))
+      !(file= heap_open(name, mode)) && my_errno == ENOENT)
   {
     HA_CREATE_INFO create_info;
     internal_table= test(test_if_locked & HA_OPEN_INTERNAL_TABLE);
@@ -142,11 +142,11 @@ int ha_heap::close(void)
   DESCRIPTION
     Do same as default implementation but use file->s->name instead of 
     table->s->path. This is needed by Windows where the clone() call sees
-    '/'-delimited path in table->s->path, while ha_heap::open() was called 
+    '/'-delimited path in table->s->path, while ha_peap::open() was called 
     with '\'-delimited path.
 */
 
-handler *ha_heap::clone(const char *name, MEM_ROOT *mem_root)
+handler *ha_heap::clone(MEM_ROOT *mem_root)
 {
   handler *new_handler= get_new_handler(table->s, mem_root, table->s->db_type());
   if (new_handler && !new_handler->ha_open(table, file->s->name, table->db_stat,
@@ -419,14 +419,6 @@ int ha_heap::delete_all_rows()
   return 0;
 }
 
-
-int ha_heap::reset_auto_increment(ulonglong value)
-{
-  file->s->auto_increment= value;
-  return 0;
-}
-
-
 int ha_heap::external_lock(THD *thd, int lock_type)
 {
   return 0;					// No external locking
