@@ -454,11 +454,9 @@ dict_load_report_deleted_index(
 /************************************************************************
 Loads definitions for index fields. */
 static
-ulint
+void
 dict_load_fields(
 /*=============*/
-				/* out: DB_SUCCESS if ok, DB_CORRUPTION 
-				if failed */
 	dict_table_t*	table,	/* in: table */
 	dict_index_t*	index,	/* in: index whose fields to load */
 	mem_heap_t*	heap)	/* in: memory heap for temporary storage */
@@ -476,7 +474,6 @@ dict_load_fields(
 	byte*		buf;
 	ulint		i;
 	mtr_t		mtr;
-	ulint		error = DB_SUCCESS;
 
 	ut_ad(mutex_own(&(dict_sys->mutex)));
 
@@ -538,26 +535,6 @@ dict_load_fields(
 
 		field = rec_get_nth_field_old(rec, 4, &len);
 
-		if (prefix_len >= DICT_MAX_INDEX_COL_LEN) {
-			fprintf(stderr, "InnoDB: Error: load index"
-					" '%s' failed.\n"
-					"InnoDB: index field '%s' has a prefix"
-					" length of %lu bytes,\n"
-					"InnoDB: which exceeds the"
-					" maximum limit of %lu bytes.\n"
-					"InnoDB: Please use server that"
-					" supports long index prefix\n"
-					"InnoDB: or turn on"
-					" innodb_force_recovery to load"
-					" the table\n",
-				index->name, mem_heap_strdupl(
-						heap, (char*) field, len),
-		    		(ulong) prefix_len,
-				(ulong) (DICT_MAX_INDEX_COL_LEN - 1));
-			error = DB_CORRUPTION;
-			goto func_exit;
-		}
-
 		dict_mem_index_add_field(index,
 					 mem_heap_strdupl(heap,
 							  (char*) field, len),
@@ -566,10 +543,8 @@ dict_load_fields(
 		btr_pcur_move_to_next_user_rec(&pcur, &mtr);
 	}
 
-func_exit:
 	btr_pcur_close(&pcur);
 	mtr_commit(&mtr);
-	return(error);
 }
 
 /************************************************************************
@@ -726,28 +701,10 @@ dict_load_indexes(
 						      space, type, n_fields);
 			index->id = id;
 
-			error = dict_load_fields(table, index, heap);
-
-			if (error != DB_SUCCESS) {
-				fprintf(stderr, "InnoDB: Error: load index '%s'"
-					" for table '%s' failed\n",
-					index->name, table->name);
-
-				/* If the force recovery flag is set, and
-				if the failed index is not the primary index, we
-				will continue and open other indexes */
-				if (srv_force_recovery
-				    && !(index->type & DICT_CLUSTERED)) {
-					error = DB_SUCCESS;
-					goto next_rec;
-				} else {
-					goto func_exit;
-				}
-			}
-
+			dict_load_fields(table, index, heap);
 			dict_index_add_to_cache(table, index, page_no);
 		}
-next_rec:
+
 		btr_pcur_move_to_next_user_rec(&pcur, &mtr);
 	}
 
@@ -907,35 +864,15 @@ err_exit:
 
 	err = dict_load_indexes(table, heap);
 
-	/* Initialize table foreign_child value. Its value could be
-	changed when dict_load_foreigns() is called below */
-	table->fk_max_recusive_level = 0;
-
 	/* If the force recovery flag is set, we open the table irrespective
 	of the error condition, since the user may want to dump data from the
 	clustered index. However we load the foreign key information only if
 	all indexes were loaded. */
-	if (err == DB_SUCCESS) {
-		err = dict_load_foreigns(table->name, TRUE, TRUE);
-
-		if (err != DB_SUCCESS) {
-			dict_table_remove_from_cache(table);
-			table = NULL;
-		} else {
-			table->fk_max_recusive_level = 0;
-		}
-	} else {
-		dict_index_t*	index;
-
-		/* Make sure that at least the clustered index was loaded.
-		Otherwise refuse to load the table */
-		index = dict_table_get_first_index(table);
-
-		if (!srv_force_recovery || !index
-		     || !(index->type & DICT_CLUSTERED)) {
-			dict_table_remove_from_cache(table);
-			table = NULL;
-		}
+	if (err != DB_SUCCESS && !srv_force_recovery) {
+		dict_mem_table_free(table);
+		table = NULL;
+	} else if (err == DB_SUCCESS) {
+		err = dict_load_foreigns(table->name, TRUE);
 	}
 #if 0
 	if (err != DB_SUCCESS && table != NULL) {
@@ -990,8 +927,6 @@ dict_load_table_on_id(
 
 	ut_ad(mutex_own(&(dict_sys->mutex)));
 
-	table = NULL;
-
 	/* NOTE that the operation of this function is protected by
 	the dictionary mutex, and therefore no deadlocks can occur
 	with other dictionary operations. */
@@ -1018,17 +953,15 @@ dict_load_table_on_id(
 				  BTR_SEARCH_LEAF, &pcur, &mtr);
 	rec = btr_pcur_get_rec(&pcur);
 
-	if (!btr_pcur_is_on_user_rec(&pcur, &mtr)) {
+	if (!btr_pcur_is_on_user_rec(&pcur, &mtr)
+	    || rec_get_deleted_flag(rec, 0)) {
 		/* Not found */
-		goto func_exit;
-	}
 
-	/* Find the first record that is not delete marked */
-	while (rec_get_deleted_flag(rec, 0)) {
-		if (!btr_pcur_move_to_next_user_rec(&pcur, &mtr)) {
-			goto func_exit;
-		}
-		rec = btr_pcur_get_rec(&pcur);
+		btr_pcur_close(&pcur);
+		mtr_commit(&mtr);
+		mem_heap_free(heap);
+
+		return(NULL);
 	}
 
 	/*---------------------------------------------------*/
@@ -1041,14 +974,19 @@ dict_load_table_on_id(
 
 	/* Check if the table id in record is the one searched for */
 	if (ut_dulint_cmp(table_id, mach_read_from_8(field)) != 0) {
-		goto func_exit;
+
+		btr_pcur_close(&pcur);
+		mtr_commit(&mtr);
+		mem_heap_free(heap);
+
+		return(NULL);
 	}
 
 	/* Now we get the table name from the record */
 	field = rec_get_nth_field_old(rec, 1, &len);
 	/* Load the table definition to memory */
 	table = dict_load_table(mem_heap_strdupl(heap, (char*) field, len));
-func_exit:
+
 	btr_pcur_close(&pcur);
 	mtr_commit(&mtr);
 	mem_heap_free(heap);
@@ -1158,12 +1096,8 @@ dict_load_foreign(
 				/* out: DB_SUCCESS or error code */
 	const char*	id,	/* in: foreign constraint id as a
 				null-terminated string */
-	ibool		check_charsets,
+	ibool		check_charsets)
 				/* in: TRUE=check charset compatibility */
-	ibool		check_recursive)
-				/* in: Whether to record the foreign table
-				parent count to avoid unlimited recursive
-				load of chained foreign tables */
 {
 	dict_foreign_t*	foreign;
 	dict_table_t*	sys_foreign;
@@ -1177,8 +1111,6 @@ dict_load_foreign(
 	ulint		len;
 	ulint		n_fields_and_type;
 	mtr_t		mtr;
-	dict_table_t*	for_table;
-	dict_table_t*	ref_table;
 
 	ut_ad(mutex_own(&(dict_sys->mutex)));
 
@@ -1263,54 +1195,11 @@ dict_load_foreign(
 
 	dict_load_foreign_cols(id, foreign);
 
-	ref_table = dict_table_check_if_in_cache_low(
-			foreign->referenced_table_name);
+	/* If the foreign table is not yet in the dictionary cache, we
+	have to load it so that we are able to make type comparisons
+	in the next function call. */
 
-	/* We could possibly wind up in a deep recursive calls if
-	we call dict_table_get_low() again here if there
-	is a chain of tables concatenated together with
-	foreign constraints. In such case, each table is
-	both a parent and child of the other tables, and
-	act as a "link" in such table chains.
-	To avoid such scenario, we would need to check the
-	number of ancesters the current table has. If that
-	exceeds DICT_FK_MAX_CHAIN_LEN, we will stop loading
-	the child table.
-	Foreign constraints are loaded in a Breath First fashion,
-	that is, the index on FOR_NAME is scanned first, and then
-	index on REF_NAME. So foreign constrains in which
-	current table is a child (foreign table) are loaded first,
-	and then those constraints where current table is a
-	parent (referenced) table.
-	Thus we could check the parent (ref_table) table's
-	reference count (fk_max_recusive_level) to know how deep the
-	recursive call is. If the parent table (ref_table) is already
-	loaded, and its fk_max_recusive_level is larger than
-	DICT_FK_MAX_CHAIN_LEN, we will stop the recursive loading
-	by skipping loading the child table. It will not affect foreign
-	constraint check for DMLs since child table will be loaded
-	at that time for the constraint check. */
-	if (!ref_table
-	    || ref_table->fk_max_recusive_level < DICT_FK_MAX_RECURSIVE_LOAD) {
-
-		/* If the foreign table is not yet in the dictionary cache, we
-		have to load it so that we are able to make type comparisons
-		in the next function call. */
-
-		for_table = dict_table_get_low(foreign->foreign_table_name);
-
-		if (for_table && ref_table && check_recursive) {
-			/* This is to record the longest chain of ancesters
-			this table has, if the parent has more ancesters
-			than this table has, record it after add 1 (for this
-			parent */
-			if (ref_table->fk_max_recusive_level
-			    >= for_table->fk_max_recusive_level) {
-				for_table->fk_max_recusive_level =
-					 ref_table->fk_max_recusive_level + 1;
-			}
-		}
-	}
+	dict_table_get_low(foreign->foreign_table_name);
 
 	/* Note that there may already be a foreign constraint object in
 	the dictionary cache for this constraint: then the following
@@ -1335,8 +1224,6 @@ dict_load_foreigns(
 /*===============*/
 					/* out: DB_SUCCESS or error code */
 	const char*	table_name,	/* in: table name */
-	ibool		check_recursive,/* in: Whether to check recursive
-					load of tables chained by FK */
 	ibool		check_charsets)	/* in: TRUE=check charset
 					compatibility */
 {
@@ -1438,7 +1325,7 @@ loop:
 
 	/* Load the foreign constraint definition to the dictionary cache */
 
-	err = dict_load_foreign(id, check_charsets, check_recursive);
+	err = dict_load_foreign(id, check_charsets);
 
 	if (err != DB_SUCCESS) {
 		btr_pcur_close(&pcur);
@@ -1466,11 +1353,6 @@ load_next_index:
 
 		mtr_start(&mtr);
 
-		/* Switch to scan index on REF_NAME, fk_max_recusive_level
-		already been updated when scanning FOR_NAME index, no need to
-		update again */
-		check_recursive = FALSE;
-
 		goto start_load;
 	}
 
