@@ -178,7 +178,8 @@ dict_table_autoinc_lock(
 /*====================*/
 	dict_table_t*	table);	/* in: table */
 /************************************************************************
-Unconditionally set the autoinc counter. */
+Initializes the autoinc counter. It is not an error to initialize an already
+initialized counter. */
 
 void
 dict_table_autoinc_initialize(
@@ -195,12 +196,12 @@ dict_table_autoinc_read(
 				/* out: value for a new row, or 0 */
 	dict_table_t*	table);	/* in: table */
 /************************************************************************
-Updates the autoinc counter if the value supplied is greater than the
-current value. */
+Updates the autoinc counter if the value supplied is equal or bigger than the
+current value. If not inited, does nothing. */
 
 void
-dict_table_autoinc_update_if_greater(
-/*=================================*/
+dict_table_autoinc_update(
+/*======================*/
 
 	dict_table_t*	table,	/* in: table */
 	ib_ulonglong	value);	/* in: value which was assigned to a row */
@@ -309,7 +310,6 @@ dict_create_foreign_constraints(
 					name before it: test.table2; the
 					default database id the database of
 					parameter name */
-	size_t		sql_length,	/* in: length of sql_string */
 	const char*	name,		/* in: table full name in the
 					normalized form
 					database_name/table_name */
