@@ -246,20 +246,22 @@ row_update_for_mysql(
 	row_prebuilt_t*	prebuilt);	/* in: prebuilt struct in MySQL
 					handle */
 /*************************************************************************
-This can only be used when srv_locks_unsafe_for_binlog is TRUE or this
-session is using a READ COMMITTED or READ UNCOMMITTED isolation level.
-Before calling this function row_search_for_mysql() must have
-initialized prebuilt->new_rec_locks to store the information which new
-record locks really were set. This function removes a newly set
-clustered index record lock under prebuilt->pcur or
-prebuilt->clust_pcur.  Thus, this implements a 'mini-rollback' that
-releases the latest clustered index record lock we set. */
+This can only be used when srv_locks_unsafe_for_binlog is TRUE or
+session is using a READ COMMITTED isolation level. Before
+calling this function we must use trx_reset_new_rec_lock_info() and
+trx_register_new_rec_lock() to store the information which new record locks
+really were set. This function removes a newly set lock under prebuilt->pcur,
+and also under prebuilt->clust_pcur. Currently, this is only used and tested
+in the case of an UPDATE or a DELETE statement, where the row lock is of the
+LOCK_X type.
+Thus, this implements a 'mini-rollback' that releases the latest record
+locks we set. */
 
 int
 row_unlock_for_mysql(
 /*=================*/
 					/* out: error code or DB_SUCCESS */
-	row_prebuilt_t*	prebuilt,	/* in/out: prebuilt struct in MySQL
+	row_prebuilt_t*	prebuilt,	/* in: prebuilt struct in MySQL
 					handle */
 	ibool		has_latches_on_recs);/* TRUE if called so that we have
 					the latches on the records under pcur
@@ -364,7 +366,6 @@ row_table_add_foreign_constraints(
 				FOREIGN KEY (a, b) REFERENCES table2(c, d),
 					table2 can be written also with the
 					database name before it: test.table2 */
-	size_t		sql_length,	/* in: length of sql_string */
 	const char*	name,		/* in: table full name in the
 					normalized form
 					database_name/table_name */
@@ -593,8 +594,6 @@ struct row_prebuilt_struct {
 	byte*		ins_upd_rec_buff;/* buffer for storing data converted
 					to the Innobase format from the MySQL
 					format */
-	const byte*	default_rec;	/* the default values of all columns
-					(a "default row") in MySQL format */
 	ulint		hint_need_to_fetch_extra_cols;
 					/* normally this is set to 0; if this
 					is set to ROW_RETRIEVE_PRIMARY_KEY,
@@ -655,20 +654,6 @@ struct row_prebuilt_struct {
 					This eliminates lock waits in some
 					cases; note that this breaks
 					serializability. */
-	ulint		new_rec_locks;	/* normally 0; if
-					srv_locks_unsafe_for_binlog is
-					TRUE or session is using READ
-					COMMITTED or READ UNCOMMITTED
-					isolation level, set in
-					row_search_for_mysql() if we set a new
-					record lock on the secondary
-					or clustered index; this is
-					used in row_unlock_for_mysql()
-					when releasing the lock under
-					the cursor if we determine
-					after retrieving the row that
-					it does not need to be locked
-					('mini-rollback') */
 	ulint		mysql_prefix_len;/* byte offset of the end of
 					the last requested column */
 	ulint		mysql_row_len;	/* length in bytes of a row in the
@@ -698,21 +683,7 @@ struct row_prebuilt_struct {
 					to this heap */
 	mem_heap_t*	old_vers_heap;	/* memory heap where a previous
 					version is built in consistent read */
-	/*----------------------*/
-	ulonglong	autoinc_last_value;/* last value of AUTO-INC interval */
-	ulonglong	autoinc_increment;/* The increment step of the auto 
-					increment column. Value must be
-					greater than or equal to 1. Required to
-					calculate the next value */
-	ulonglong	autoinc_offset; /* The offset passed to
-					get_auto_increment() by MySQL. Required
-					to calculate the next value */
-	ulint		autoinc_error;	/* The actual error code encountered
-					while trying to init or read the
-					autoinc value from the table. We
-					store it here so that we can return
-					it to MySQL */
-	/*----------------------*/
+	ulonglong	last_value;	/* last value of AUTO-INC interval */
 	ulint		magic_n2;	/* this should be the same as
 					magic_n */
 };
