@@ -131,8 +131,6 @@ trx_create(
 
 	trx->mysql_thd = NULL;
 	trx->mysql_query_str = NULL;
-	trx->mysql_query_len = NULL;
-
 	trx->active_trans = 0;
 	trx->duplicates = 0;
 
@@ -194,6 +192,8 @@ trx_create(
 
 	trx->n_autoinc_rows = 0;
 
+	trx_reset_new_rec_lock_info(trx);
+
 	return(trx);
 }
 
@@ -287,10 +287,6 @@ trx_free(
 		      "InnoDB: inside InnoDB.\n", stderr);
 		trx_print(stderr, trx, 600);
 		putc('\n', stderr);
-
-		/* This is an error but not a fatal error. We must keep
-		the counters like srv_conc_n_threads accurate. */
-		srv_conc_force_exit_innodb(trx);
 	}
 
 	if (trx->n_mysql_tables_in_use != 0
@@ -930,15 +926,14 @@ trx_commit_off_kernel(
 		mutex_enter(&kernel_mutex);
 	}
 
-	/* Free all savepoints */
-	trx_roll_free_all_savepoints(trx);
+	/* Free savepoints */
+	trx_roll_savepoints_free(trx, NULL);
 
 	trx->conc_state = TRX_NOT_STARTED;
 	trx->rseg = NULL;
 	trx->undo_no = ut_dulint_zero;
 	trx->last_sql_stat_start.least_undo_no = ut_dulint_zero;
 	trx->mysql_query_str = NULL;
-	trx->mysql_query_len = NULL;
 
 	ut_ad(UT_LIST_GET_LEN(trx->wait_thrs) == 0);
 	ut_ad(UT_LIST_GET_LEN(trx->trx_locks) == 0);
@@ -1655,7 +1650,9 @@ trx_mark_sql_stat_end(
 
 /**************************************************************************
 Prints info about a transaction to the given file. The caller must own the
-kernel mutex. */
+kernel mutex and must have called
+innobase_mysql_prepare_print_arbitrary_thd(), unless he knows that MySQL
+or InnoDB cannot meanwhile change the info printed here. */
 
 void
 trx_print(
