@@ -161,11 +161,11 @@ static int FTB_WORD_cmp(my_off_t *v, FTB_WORD *a, FTB_WORD *b)
 
 static int FTB_WORD_cmp_list(CHARSET_INFO *cs, FTB_WORD **a, FTB_WORD **b)
 {
-  /* ORDER BY word, ndepth */
-  int i= ha_compare_text(cs, (uchar*) (*a)->word + 1, (*a)->len - 1,
-                             (uchar*) (*b)->word + 1, (*b)->len - 1, 0, 0);
+  /* ORDER BY word DESC, ndepth DESC */
+  int i= ha_compare_text(cs, (uchar*) (*b)->word+1,(*b)->len-1,
+                             (uchar*) (*a)->word+1,(*a)->len-1,0,0);
   if (!i)
-    i= CMP_NUM((*a)->ndepth, (*b)->ndepth);
+    i=CMP_NUM((*b)->ndepth,(*a)->ndepth);
   return i;
 }
 
@@ -335,32 +335,17 @@ static int _ftb_no_dupes_cmp(void* not_used __attribute__((unused)),
   return CMP_NUM((*((my_off_t*)a)), (*((my_off_t*)b)));
 }
 
-/*
-  When performing prefix search (a word with truncation operator), we
-  must preserve original prefix to ensure that characters which may be
-  expanded/contracted do not break the prefix. This is done by storing
-  newly found key immediatly after the original word in ftbw->word
-  buffer.
-
-  ftbw->word= LENGTH WORD [ LENGTH1 WORD1 ] WEIGHT REFERENCE
-  LENGTH - 1 byte, length of the WORD
-  WORD - LENGTH bytes, the word itself
-  LENGTH1 - 1 byte, length of the WORD1, present in case of prefix search
-  WORD1 - LENGTH bytes, the word itself, present in case of prefix search
-  WEIGHT - 4 bytes (HA_FT_WLEN), either weight or number of subkeys
-  REFERENCE - rec_reflength bytes, pointer to the record
-
-  returns 1 if the search was finished (must-word wasn't found)
-*/
+/* returns 1 if the search was finished (must-word wasn't found) */
 static int _ft2_search(FTB *ftb, FTB_WORD *ftbw, my_bool init_search)
 {
   int r;
   int subkeys=1;
   my_bool can_go_down;
   MI_INFO *info=ftb->info;
-  uint UNINIT_VAR(off), extra=HA_FT_WLEN+info->s->base.rec_reflength;
+  uint off, extra=HA_FT_WLEN+info->s->base.rec_reflength;
   uchar *lastkey_buf=ftbw->word+ftbw->off;
 
+  LINT_INIT(off);
   if (ftbw->flags & FTB_FLAG_TRUNC)
     lastkey_buf+=ftbw->len;
 
@@ -384,8 +369,7 @@ static int _ft2_search(FTB *ftb, FTB_WORD *ftbw, my_bool init_search)
     if (ftbw->docid[0] < max_docid)
     {
       sflag|= SEARCH_SAME;
-      _mi_dpointer(info, (uchar*) (lastkey_buf + HA_FT_WLEN +
-                                   (ftbw->off ? 0 : lastkey_buf[0] + 1)),
+      _mi_dpointer(info, (uchar *)(ftbw->word + ftbw->len + HA_FT_WLEN),
                    max_docid);
     }
     r=_mi_search(info, ftbw->keyinfo, (uchar*) lastkey_buf,
@@ -437,18 +421,8 @@ static int _ft2_search(FTB *ftb, FTB_WORD *ftbw, my_bool init_search)
         return 0;
     }
 
-    /*
-      Going up to the first-level tree to continue search there.
-      Only done when performing prefix search.
-
-      Key buffer data pointer as well as docid[0] may be smaller
-      than values we got while searching first-level tree. Thus
-      they must be restored to original values to avoid dead-loop,
-      when subsequent search for a bigger value eventually ends up
-      in this same second-level tree.
-    */
+    /* going up to the first-level tree to continue search there */
     _mi_dpointer(info, (uchar*) (lastkey_buf+HA_FT_WLEN), ftbw->key_root);
-    ftbw->docid[0]= ftbw->key_root;
     ftbw->key_root=info->s->state.key_root[ftb->keynr];
     ftbw->keyinfo=info->s->keyinfo+ftb->keynr;
     ftbw->off=0;
@@ -485,7 +459,8 @@ static void _ftb_init_index_search(FT_INFO *ftb)
   int i;
   FTB_WORD   *ftbw;
 
-  if (ftb->state == UNINITIALIZED || ftb->keynr == NO_SUCH_KEY)
+  if ((ftb->state != READY && ftb->state !=INDEX_DONE) ||
+      ftb->keynr == NO_SUCH_KEY)
     return;
   ftb->state=INDEX_SEARCH;
 
@@ -890,49 +865,23 @@ static int ftb_find_relevance_add_word(MYSQL_FTPARSER_PARAM *param,
   FT_INFO *ftb= ftb_param->ftb;
   FTB_WORD *ftbw;
   int a, b, c;
-  /*
-    Find right-most element in the array of query words matching this
-    word from a document.
-  */
   for (a= 0, b= ftb->queue.elements, c= (a+b)/2; b-a>1; c= (a+b)/2)
   {
     ftbw= ftb->list[c];
     if (ha_compare_text(ftb->charset, (uchar*)word, len,
                         (uchar*)ftbw->word+1, ftbw->len-1,
-                        (my_bool) (ftbw->flags & FTB_FLAG_TRUNC), 0) < 0)
+                        (my_bool)(ftbw->flags&FTB_FLAG_TRUNC), 0) > 0)
       b= c;
     else
       a= c;
   }
-  /*
-    If there were no words with truncation operator, we iterate to the
-    beginning of an array until array element is equal to the word from
-    a document. This is done mainly because the same word may be
-    mentioned twice (or more) in the query.
-
-    In case query has words with truncation operator we must iterate
-    to the beginning of the array. There may be non-matching query words
-    between matching word with truncation operator and the right-most
-    matching element. E.g., if we're looking for 'aaa15' in an array of
-    'aaa1* aaa14 aaa15 aaa16'.
-
-    Worse of that there still may be match even if the binary search
-    above didn't find matching element. E.g., if we're looking for
-    'aaa15' in an array of 'aaa1* aaa14 aaa16'. The binary search will
-    stop at 'aaa16'.
-  */
   for (; c >= 0; c--)
   {
     ftbw= ftb->list[c];
     if (ha_compare_text(ftb->charset, (uchar*)word, len,
                         (uchar*)ftbw->word + 1,ftbw->len - 1,
                         (my_bool)(ftbw->flags & FTB_FLAG_TRUNC), 0))
-    {
-      if (ftb->with_scan & FTB_FLAG_TRUNC)
-        continue;
-      else
-        break;
-    }
+      break;
     if (ftbw->docid[1] == ftb->info->lastpos)
       continue;
     ftbw->docid[1]= ftb->info->lastpos;
