@@ -128,7 +128,7 @@ join_buffer_size = 8M
 # the amount of thread creations needed if you have a lot of new
 # connections. (Normally this doesn't give a notable performance
 # improvement if you have a good thread implementation.)
-thread_cache = 8
+thread_cache_size = 8
 
 # This permits the application to give the threads system a hint for the
 # desired number of threads that should be run at the same time.  This
@@ -191,6 +191,9 @@ tmp_table_size = 64M
 # the ability to do point in time recovery from your latest backup.
 log-bin=mysql-bin
 
+# binary logging format - mixed recommended
+binlog_format=mixed
+
 # If you're using replication with chained slaves (A->B->C), you need to
 # enable this option on server B. It enables logging of updates done by
 # the slave thread into the slave's binary log.
@@ -357,14 +360,6 @@ myisam_repair_threads = 1
 myisam_recover
 
 
-# *** BDB Specific options ***
-
-# Use this option if you run a MySQL server with BDB support enabled but
-# you do not plan to use it. This will save memory and may speed up some
-# things.
-skip-bdb
-
-
 # *** INNODB Specific options ***
 
 # Use this option if you have a MySQL server with InnoDB support enabled
