@@ -4,11 +4,11 @@
 # an important part, or systems up to 128M where MySQL is used together with
 # other programs (such as a web server)
 #
-# MySQL programs look for option files in a set of
-# locations which depend on the deployment platform.
-# You can copy this option file to one of those
-# locations. For information about these locations, see:
-# http://dev.mysql.com/doc/mysql/en/option-files.html
+# You can copy this file to
+# /etc/my.cnf to set global options,
+# mysql-data-dir/my.cnf to set server-specific options (in this
+# installation this directory is @localstatedir@) or
+# ~/.my.cnf to set user-specific options.
 #
 # In this file, you can use all long options that a program supports.
 # If you want to know which options a program supports, run the program
@@ -27,9 +27,9 @@ socket		= @MYSQL_UNIX_ADDR@
 port		= @MYSQL_TCP_PORT@
 socket		= @MYSQL_UNIX_ADDR@
 skip-locking
-key_buffer_size = 16M
+key_buffer = 16M
 max_allowed_packet = 1M
-table_open_cache = 64
+table_cache = 64
 sort_buffer_size = 512K
 net_buffer_length = 8K
 read_buffer_size = 256K
@@ -44,6 +44,9 @@ myisam_sort_buffer_size = 8M
 # 
 #skip-networking
 
+# Disable Federated by default
+skip-federated
+
 # Replication Master Server (default)
 # binary logging is required for replication
 log-bin=mysql-bin
@@ -111,10 +114,15 @@ server-id	= 1
 # binary logging - not required for slaves, but recommended
 #log-bin=mysql-bin
 
+# Point the following paths to different dedicated disks
+#tmpdir		= /tmp/		
+#log-update 	= /path-to-dedicated-directory/hostname
+
 # Uncomment the following if you are using InnoDB tables
-#innodb_data_home_dir = @localstatedir@
+#innodb_data_home_dir = @localstatedir@/
 #innodb_data_file_path = ibdata1:10M:autoextend
-#innodb_log_group_home_dir = @localstatedir@
+#innodb_log_group_home_dir = @localstatedir@/
+#innodb_log_arch_dir = @localstatedir@/
 # You can set .._buffer_pool_size up to 50 - 80 %
 # of RAM but beware of setting memory usage too high
 #innodb_buffer_pool_size = 16M
@@ -134,8 +142,14 @@ no-auto-rehash
 # Remove the next comment character if you are not familiar with SQL
 #safe-updates
 
+[isamchk]
+key_buffer = 20M
+sort_buffer_size = 20M
+read_buffer = 2M
+write_buffer = 2M
+
 [myisamchk]
-key_buffer_size = 20M
+key_buffer = 20M
 sort_buffer_size = 20M
 read_buffer = 2M
 write_buffer = 2M
